@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -27,12 +27,18 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#include <limits.h>
+#include "base/util.h"
+
+#include <climits>
+#include <cstdlib>
+#include <ctime>
 #include <fstream>
 #include <sstream>
 #include <string>
+
+#include "base/clock_mock.h"
 #include "base/file_stream.h"
-#include "base/util.h"
+#include "base/mmap.h"
 #include "base/mutex.h"
 #include "base/thread.h"
 #include "testing/base/public/googletest.h"
@@ -230,6 +236,49 @@ TEST(UtilTest, CapitalizeString) {
             "\xef\xbd\x98\xef\xbd\x99\xef\xbd\x9a\xef\xbd\x9b", s2);
 }
 
+void VerifyUTF8ToUCS4(const string &text, char32 expected_ucs4,
+                      size_t expected_len) {
+  const char *begin = text.data();
+  const char *end = begin + text.size();
+  size_t mblen = 0;
+  char32 result = Util::UTF8ToUCS4(begin, end, &mblen);
+  EXPECT_EQ(expected_ucs4, result) << text << " " << expected_ucs4;
+  EXPECT_EQ(expected_len, mblen) << text << " " << expected_len;
+}
+
+TEST(UtilTest, UTF8ToUCS4) {
+  VerifyUTF8ToUCS4("", 0, 0);
+  VerifyUTF8ToUCS4("\x01", 1, 1);
+  VerifyUTF8ToUCS4("\x7F", 0x7F, 1);
+  VerifyUTF8ToUCS4("\xC2\x80", 0x80, 2);
+  VerifyUTF8ToUCS4("\xDF\xBF", 0x7FF, 2);
+  VerifyUTF8ToUCS4("\xE0\xA0\x80", 0x800, 3);
+  VerifyUTF8ToUCS4("\xEF\xBF\xBF", 0xFFFF, 3);
+  VerifyUTF8ToUCS4("\xF0\x90\x80\x80", 0x10000, 4);
+  VerifyUTF8ToUCS4("\xF7\xBF\xBF\xBF", 0x1FFFFF, 4);
+  // do not test 5-6 bytes because it's out of spec of UTF8.
+}
+
+TEST(UtilTest, UCS4ToUTF8) {
+  string output;
+  Util::UCS4ToUTF8(0, &output);
+  EXPECT_EQ("\0", output);
+  Util::UCS4ToUTF8(0x7F, &output);
+  EXPECT_EQ("\x7F", output);
+  Util::UCS4ToUTF8(0x80, &output);
+  EXPECT_EQ("\xC2\x80", output);
+  Util::UCS4ToUTF8(0x7FF, &output);
+  EXPECT_EQ("\xDF\xBF", output);
+  Util::UCS4ToUTF8(0x800, &output);
+  EXPECT_EQ("\xE0\xA0\x80", output);
+  Util::UCS4ToUTF8(0xFFFF, &output);
+  EXPECT_EQ("\xEF\xBF\xBF", output);
+  Util::UCS4ToUTF8(0x10000, &output);
+  EXPECT_EQ("\xF0\x90\x80\x80", output);
+  Util::UCS4ToUTF8(0x1FFFFF, &output);
+  EXPECT_EQ("\xF7\xBF\xBF\xBF", output);
+}
+
 TEST(UtilTest, CharsLen) {
   // "私の名前は中野です"
   const string src = "\xe7\xa7\x81\xe3\x81\xae\xe5\x90\x8d\xe5\x89\x8d\xe3\x81"
@@ -371,10 +420,16 @@ TEST(UtilTest, SafeStrToUInt32) {
 
   EXPECT_TRUE(Util::SafeStrToUInt32("0", &value));
   EXPECT_EQ(0, value);
+
+  value = 0xDEADBEEF;
   EXPECT_TRUE(Util::SafeStrToUInt32(" \t\r\n\v\f0 \t\r\n\v\f", &value));
   EXPECT_EQ(0, value);
+
+  value = 0xDEADBEEF;
   EXPECT_TRUE(Util::SafeStrToUInt32("012345678", &value));
   EXPECT_EQ(12345678, value);
+
+  value = 0xDEADBEEF;
   EXPECT_TRUE(Util::SafeStrToUInt32("4294967295", &value));
   EXPECT_EQ(4294967295u, value);  // max of 32-bit unsigned integer
 
@@ -387,6 +442,66 @@ TEST(UtilTest, SafeStrToUInt32) {
   EXPECT_FALSE(Util::SafeStrToUInt32("", &value));
 }
 
+TEST(UnitTest, SafeHexStrToUInt32) {
+  uint32 value = 0xDEADBEEF;
+
+  EXPECT_TRUE(Util::SafeHexStrToUInt32("0", &value));
+  EXPECT_EQ(0, value);
+
+  value = 0xDEADBEEF;
+  EXPECT_TRUE(Util::SafeHexStrToUInt32(" \t\r\n\v\f0 \t\r\n\v\f", &value));
+  EXPECT_EQ(0, value);
+
+  value = 0xDEADBEEF;
+  EXPECT_TRUE(Util::SafeHexStrToUInt32("0ABCDE", &value));
+  EXPECT_EQ(0xABCDE, value);
+
+  value = 0xDEADBEEF;
+  EXPECT_TRUE(Util::SafeHexStrToUInt32("0abcde", &value));
+  EXPECT_EQ(0xABCDE, value);
+
+  value = 0xDEADBEEF;
+  EXPECT_TRUE(Util::SafeHexStrToUInt32("FFFFFFFF", &value));
+  EXPECT_EQ(0xFFFFFFFF, value);  // max of 32-bit unsigned integer
+
+  value = 0xDEADBEEF;
+  EXPECT_TRUE(Util::SafeHexStrToUInt32("ffffffff", &value));
+  EXPECT_EQ(0xFFFFFFFF, value);  // max of 32-bit unsigned integer
+
+  EXPECT_FALSE(Util::SafeHexStrToUInt32("-0", &value));
+  EXPECT_FALSE(Util::SafeHexStrToUInt32("100000000", &value));  // overflow
+  EXPECT_FALSE(Util::SafeHexStrToUInt32("GHIJK", &value));
+  EXPECT_FALSE(Util::SafeHexStrToUInt32("0.", &value));
+  EXPECT_FALSE(Util::SafeHexStrToUInt32(".0", &value));
+  EXPECT_FALSE(Util::SafeHexStrToUInt32("", &value));
+}
+
+TEST(UnitTest, SafeOctStrToUInt32) {
+  uint32 value = 0xDEADBEEF;
+
+  EXPECT_TRUE(Util::SafeOctStrToUInt32("0", &value));
+  EXPECT_EQ(0, value);
+
+  value = 0xDEADBEEF;
+  EXPECT_TRUE(Util::SafeOctStrToUInt32(" \t\r\n\v\f0 \t\r\n\v\f", &value));
+  EXPECT_EQ(0, value);
+
+  value = 0xDEADBEEF;
+  EXPECT_TRUE(Util::SafeOctStrToUInt32("012345", &value));
+  EXPECT_EQ(012345, value);
+
+  value = 0xDEADBEEF;
+  EXPECT_TRUE(Util::SafeOctStrToUInt32("37777777777", &value));
+  EXPECT_EQ(0xFFFFFFFF, value);  // max of 32-bit unsigned integer
+
+  EXPECT_FALSE(Util::SafeOctStrToUInt32("-0", &value));
+  EXPECT_FALSE(Util::SafeOctStrToUInt32("40000000000", &value));  // overflow
+  EXPECT_FALSE(Util::SafeOctStrToUInt32("9AB", &value));
+  EXPECT_FALSE(Util::SafeOctStrToUInt32("0.", &value));
+  EXPECT_FALSE(Util::SafeOctStrToUInt32(".0", &value));
+  EXPECT_FALSE(Util::SafeOctStrToUInt32("", &value));
+}
+
 TEST(UtilTest, SafeStrToUInt64) {
   uint64 value = 0xDEADBEEF;
 
@@ -450,10 +565,12 @@ TEST(UtilTest, SafeStrToDouble) {
 #endif
 }
 
+#ifdef __GNUC__
+#pragma GCC diagnostic ignored "-Wformat-zero-length"
+#endif  // __GNUC__
 TEST(UtilTest, StringPrintf) {
   // strings
-  // NB: GCC warns an empty format string, so uses string().c_str() instead.
-  EXPECT_EQ("", Util::StringPrintf(string().c_str()));
+  EXPECT_EQ("", Util::StringPrintf(""));
   EXPECT_EQ("", Util::StringPrintf("%s", ""));
   EXPECT_EQ("hello, world", Util::StringPrintf("hello, world"));
   EXPECT_EQ("hello, world", Util::StringPrintf("%s", "hello, world"));
@@ -1076,6 +1193,26 @@ TEST(UtilTest, NormalizeNumbers) {
   }
 }
 
+TEST(UtilTest, IsEnglishTransliteration) {
+  EXPECT_TRUE(Util::IsEnglishTransliteration("ABC"));
+  EXPECT_TRUE(Util::IsEnglishTransliteration("Google"));
+  EXPECT_TRUE(Util::IsEnglishTransliteration("Google Map"));
+  EXPECT_TRUE(Util::IsEnglishTransliteration("ABC-DEF"));
+  EXPECT_TRUE(Util::IsEnglishTransliteration("Foo-bar"));
+  EXPECT_TRUE(Util::IsEnglishTransliteration("Foo!"));
+  EXPECT_TRUE(Util::IsEnglishTransliteration("!"));
+  EXPECT_TRUE(Util::IsEnglishTransliteration("  "));
+  //  EXPECT_FALSE(Util::IsEnglishTransliteration("てすと"));
+  //  EXPECT_FALSE(Util::IsEnglishTransliteration("テスト"));
+  //  EXPECT_FALSE(Util::IsEnglishTransliteration("東京"));
+  EXPECT_FALSE(Util::IsEnglishTransliteration(
+      "\xE3\x81\xA6\xE3\x81\x99\xE3\x81\xA8"));
+  EXPECT_FALSE(Util::IsEnglishTransliteration(
+      "\xE3\x83\x86\xE3\x82\xB9\xE3\x83\x88"));
+  EXPECT_FALSE(Util::IsEnglishTransliteration(
+      "\xE6\x9D\xB1\xE4\xBA\xAC"));
+}
+
 TEST(UtilTest, Basename) {
 #ifdef OS_WINDOWS
   EXPECT_EQ("bar", Util::Basename("\\foo\\bar"));
@@ -1171,6 +1308,79 @@ TEST(UtilTest, ChopReturns) {
   EXPECT_EQ("line", line);
 }
 
+// 2020-12-23 13:24:35 (Wed) UTC
+// 123456 [usec]
+const uint64 kTestSeconds = 1608729875uLL;
+const uint32 kTestMicroSeconds = 123456u;
+
+// time utility test with mock clock
+TEST(UtilTest, TimeTestWithMock) {
+  scoped_ptr<ClockMock> mock_clock(
+      new ClockMock(kTestSeconds, kTestMicroSeconds));
+  Util::SetClockHandler(mock_clock.get());
+
+  // GetTime,
+  {
+    EXPECT_EQ(kTestSeconds, Util::GetTime());
+  }
+
+  // GetTimeOfDay
+  {
+    uint64 current_sec;
+    uint32 current_usec;
+    Util::GetTimeOfDay(&current_sec, &current_usec);
+    EXPECT_EQ(kTestSeconds, current_sec);
+    EXPECT_EQ(kTestMicroSeconds, current_usec);
+  }
+
+  // GetCurrentTm
+  // 2020-12-23 13:24:35 (Wed)
+  {
+    tm current_tm;
+    Util::GetCurrentTm(&current_tm);
+    EXPECT_EQ(120, current_tm.tm_year);
+    EXPECT_EQ(11,  current_tm.tm_mon);
+    EXPECT_EQ(23,  current_tm.tm_mday);
+    EXPECT_EQ(13,  current_tm.tm_hour);
+    EXPECT_EQ(24,  current_tm.tm_min);
+    EXPECT_EQ(35,  current_tm.tm_sec);
+    EXPECT_EQ(3,   current_tm.tm_wday);
+  }
+
+  // GetTmWithoutOffsetSecond
+  // 2024/02/23 23:11:15 (Fri)
+  {
+    const int offset_seconds = 100000000;
+    tm offset_tm;
+    Util::GetTmWithOffsetSecond(&offset_tm, offset_seconds);
+    EXPECT_EQ(124, offset_tm.tm_year);
+    EXPECT_EQ(1,   offset_tm.tm_mon);
+    EXPECT_EQ(23,  offset_tm.tm_mday);
+    EXPECT_EQ(23,  offset_tm.tm_hour);
+    EXPECT_EQ(11,  offset_tm.tm_min);
+    EXPECT_EQ(15,  offset_tm.tm_sec);
+    EXPECT_EQ(5,   offset_tm.tm_wday);
+  }
+
+  // unset clock handler
+  Util::SetClockHandler(NULL);
+}
+
+// time utility test without mock clock
+TEST(UtilTest, TimeTestWithoutMock) {
+  uint64 get_time_of_day_sec, get_time_sec;
+  uint32 get_time_of_day_usec;
+
+  Util::GetTimeOfDay(&get_time_of_day_sec, &get_time_of_day_usec);
+  get_time_sec = Util::GetTime();
+
+  // hmm, unstable test.
+  const int margin = 1;
+  EXPECT_NEAR(get_time_of_day_sec, get_time_sec, margin)
+      << ": This test have possibilities to fail "
+      << "when system is busy and slow.";
+}
+
 // Initialize argc and argv for unittest.
 class Arguments {
  public:
@@ -1349,6 +1559,13 @@ TEST(UtilTest, Escape) {
   EXPECT_EQ("\\xE3\\x82\\x89\\xE3\\x82\\x80\\xE3\\x81\\xA0", escaped);
 }
 
+TEST(UtilTest, EscapeUrl) {
+  string escaped;
+  // "らむだ"
+  Util::EscapeUrl("\xe3\x82\x89\xe3\x82\x80\xe3\x81\xa0", &escaped);
+  EXPECT_EQ("%E3%82%89%E3%82%80%E3%81%A0", escaped);
+}
+
 TEST(UnitTest, EscapeHtml) {
   string escaped;
   Util::EscapeHtml("<>&'\"abc", &escaped);
@@ -1367,6 +1584,8 @@ TEST(UtilTest, ScriptType) {
                                  Util::HIRAGANA));
   // "京都"
   EXPECT_TRUE(Util::IsScriptType("\xe4\xba\xac\xe9\x83\xbd", Util::KANJI));
+  // "人々" (b/4201140)
+  EXPECT_TRUE(Util::IsScriptType("\xE4\xBA\xBA\xE3\x80\x85", Util::KANJI));
   // "モズク"
   EXPECT_TRUE(Util::IsScriptType("\xe3\x83\xa2\xe3\x82\xba\xe3\x82\xaf",
                                  Util::KATAKANA));
@@ -1380,6 +1599,21 @@ TEST(UtilTest, ScriptType) {
   // "グーグル"
   EXPECT_TRUE(Util::IsScriptType("\xe3\x82\xb0\xe3\x83\xbc\xe3\x82\xb0\xe3\x83"
                                  "\xab", Util::KATAKANA));
+  // "ゟ" U+309F: HIRAGANA DIGRAPH YORI
+  EXPECT_TRUE(Util::IsScriptType("\xE3\x82\x9F", Util::HIRAGANA));
+  // "ヿ" U+30FF: KATAKANA DIGRAPH KOTO
+  EXPECT_TRUE(Util::IsScriptType("\xE3\x83\xBF", Util::KATAKANA));
+  // "ヷヸヹヺㇰㇱㇲㇳㇴㇵㇶㇷㇸㇹㇺㇻㇼㇽㇾㇿ"
+  EXPECT_TRUE(Util::IsScriptType(
+      "\xE3\x83\xB7\xE3\x83\xB8\xE3\x83\xB9\xE3\x83\xBA\xE3\x87\xB0"
+      "\xE3\x87\xB1\xE3\x87\xB2\xE3\x87\xB3\xE3\x87\xB4\xE3\x87\xB5"
+      "\xE3\x87\xB6\xE3\x87\xB7\xE3\x87\xB8\xE3\x87\xB9\xE3\x87\xBA"
+      "\xE3\x87\xBB\xE3\x87\xBC\xE3\x87\xBD\xE3\x87\xBE\xE3\x87\xBF",
+      Util::KATAKANA));
+  // "𛀀" U+1B000: KATAKANA LETTER ARCHAIC E
+  EXPECT_TRUE(Util::IsScriptType("\xF0\x9B\x80\x80", Util::KATAKANA));
+  // "𛀁" U+1B001: HIRAGANA LETTER ARCHAIC YE
+  EXPECT_TRUE(Util::IsScriptType("\xF0\x9B\x80\x81", Util::HIRAGANA));
 
   EXPECT_TRUE(Util::IsScriptType("012", Util::NUMBER));
   // "０１２012"
@@ -1437,6 +1671,8 @@ TEST(UtilTest, ScriptType) {
                                                 "\x81\x86"));
   // "京都"
   EXPECT_EQ(Util::KANJI, Util::GetScriptType("\xe4\xba\xac\xe9\x83\xbd"));
+  // "人々" (b/4201140)
+  EXPECT_EQ(Util::KANJI, Util::GetScriptType("\xE4\xBA\xBA\xE3\x80\x85"));
   // "モズク"
   EXPECT_EQ(Util::KATAKANA, Util::GetScriptType("\xe3\x83\xa2\xe3\x82\xba\xe3"
                                                 "\x82\xaf"));
@@ -1451,6 +1687,21 @@ TEST(UtilTest, ScriptType) {
   // "グーグル"
   EXPECT_EQ(Util::KATAKANA, Util::GetScriptType("\xe3\x82\xb0\xe3\x83\xbc\xe3"
                                                 "\x82\xb0\xe3\x83\xab"));
+  // "ゟ" U+309F HIRAGANA DIGRAPH YORI
+  EXPECT_EQ(Util::HIRAGANA, Util::GetScriptType("\xE3\x82\x9F"));
+  // "ヿ" U+30FF KATAKANA DIGRAPH KOTO
+  EXPECT_EQ(Util::KATAKANA, Util::GetScriptType("\xE3\x83\xBF"));
+  // "ヷヸヹヺㇰㇱㇲㇳㇴㇵㇶㇷㇸㇹㇺㇻㇼㇽㇾㇿ"
+  EXPECT_EQ(Util::KATAKANA, Util::GetScriptType(
+      "\xE3\x83\xB7\xE3\x83\xB8\xE3\x83\xB9\xE3\x83\xBA\xE3\x87\xB0"
+      "\xE3\x87\xB1\xE3\x87\xB2\xE3\x87\xB3\xE3\x87\xB4\xE3\x87\xB5"
+      "\xE3\x87\xB6\xE3\x87\xB7\xE3\x87\xB8\xE3\x87\xB9\xE3\x87\xBA"
+      "\xE3\x87\xBB\xE3\x87\xBC\xE3\x87\xBD\xE3\x87\xBE\xE3\x87\xBF"));
+  // "𛀀" U+1B000 KATAKANA LETTER ARCHAIC E
+  EXPECT_EQ(Util::KATAKANA, Util::GetScriptType("\xF0\x9B\x80\x80"));
+  // "𛀁" U+1B001 HIRAGANA LETTER ARCHAIC YE
+  EXPECT_EQ( Util::HIRAGANA, Util::GetScriptType("\xF0\x9B\x80\x81"));
+
   // "!グーグル"
   EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptType("\x21\xe3\x82\xb0\xe3\x83"
                                                       "\xbc\xe3\x82\xb0\xe3\x83"
@@ -1555,6 +1806,19 @@ TEST(UtilTest, ScriptType) {
   EXPECT_EQ(Util::UNKNOWN_SCRIPT, Util::GetScriptType("\xe3\x82\xb0\xe3\x83\xbc"
                                                       "\xe3\x82\xb0\xe3\x83\xab"
                                                       "\xe3\x81\x90"));
+
+  // "龦" U+9FA6
+  EXPECT_EQ(Util::KANJI, Util::GetScriptType("\xE9\xBE\xA6"));
+  // "龻" U+9FBB
+  EXPECT_EQ(Util::KANJI, Util::GetScriptType("\xE9\xBE\xBB"));
+  // U+9FFF is not assigned yet but reserved for CJK Unified Ideographs.
+  EXPECT_EQ(Util::KANJI, Util::GetScriptType("\xE9\xBE\xFF"));
+  // "𠮟咤" U+20B9F U+54A4
+  EXPECT_EQ(Util::KANJI, Util::GetScriptType("\xF0\xA0\xAE\x9F\xE5\x92\xA4"));
+  // "𠮷野" U+20BB7 U+91CE
+  EXPECT_EQ(Util::KANJI, Util::GetScriptType("\xF0\xA0\xAE\xB7\xE9\x87\x8E"));
+  // "巽" U+2F884
+  EXPECT_EQ(Util::KANJI, Util::GetScriptType("\xF0\xAF\xA2\x84"));
 }
 
 
@@ -1680,6 +1944,40 @@ TEST(UtilTest, FormType) {
   EXPECT_EQ(Util::HALF_WIDTH, Util::GetFormType("@!#"));
 }
 
+
+TEST(UtilTest, CharacterSet_gen_character_set) {
+  // [0x00, 0x7f] are ASCII
+  for (size_t i = 0; i <= 0x7f; ++i) {
+    EXPECT_EQ(Util::ASCII, Util::GetCharacterSet(i));
+  }
+  // [0x80, 0xff] are not ASCII
+  for (size_t i = 0x80; i <= 0xff; ++i) {
+    EXPECT_NE(Util::ASCII, Util::GetCharacterSet(i));
+  }
+
+  // 0213
+  // "Ⅰ"
+  EXPECT_EQ(Util::JISX0213, Util::GetCharacterSet(0x2160));
+  // "①"
+  EXPECT_EQ(Util::JISX0213, Util::GetCharacterSet(0x2460));
+  // "㊤"
+  EXPECT_EQ(Util::JISX0213, Util::GetCharacterSet(0x32A4));
+  // "𠮟" from UCS4 ragne (b/4176888)
+  EXPECT_EQ(Util::JISX0213, Util::GetCharacterSet(0x20B9F));
+  // "𪚲" from UCS4 ragne (b/4176888)
+  EXPECT_EQ(Util::JISX0213, Util::GetCharacterSet(0x2A6B2));
+
+  // only in CP932
+  // "凬"
+  EXPECT_EQ(Util::CP932, Util::GetCharacterSet(0x51EC));
+
+  // only in Unicode
+  // "￦"
+  EXPECT_EQ(Util::UNICODE_ONLY, Util::GetCharacterSet(0xFFE6));
+  // "𠮷" from UCS4 ragne (b/4176888)
+  EXPECT_EQ(Util::UNICODE_ONLY, Util::GetCharacterSet(0x20BB7));
+}
+
 TEST(UtilTest, CharacterSet) {
   // "あいうえお"
   EXPECT_EQ(Util::JISX0208, Util::GetCharacterSet("\xe3\x81\x82\xe3\x81\x84\xe3"
@@ -1707,6 +2005,10 @@ TEST(UtilTest, CharacterSet) {
   EXPECT_EQ(Util::JISX0213, Util::GetCharacterSet("\xe2\x91\xa0"));
   // "㊤"
   EXPECT_EQ(Util::JISX0213, Util::GetCharacterSet("\xe3\x8a\xa4"));
+  // "𠮟" from UCS4 ragne (b/4176888)
+  EXPECT_EQ(Util::JISX0213, Util::GetCharacterSet("\xF0\xA0\xAE\x9F"));
+  // "𪚲" from UCS4 ragne (b/4176888)
+  EXPECT_EQ(Util::JISX0213, Util::GetCharacterSet("\xF0\xAA\x9A\xB2"));
 
   // only in CP932
   // "凬"
@@ -1715,6 +2017,8 @@ TEST(UtilTest, CharacterSet) {
   // only in Unicode
   // "￦"
   EXPECT_EQ(Util::UNICODE_ONLY, Util::GetCharacterSet("\xef\xbf\xa6"));
+  // "𠮷" from UCS4 ragne (b/4176888)
+  EXPECT_EQ(Util::UNICODE_ONLY, Util::GetCharacterSet("\xF0\xA0\xAE\xB7"));
 }
 
 TEST(UtilTest, WriteByteArray) {
@@ -1870,6 +2174,16 @@ TEST(UtilTest, GetFileVersionStringTest) {
   ASSERT_TRUE(Util::SafeStrToUInt32(numbers[3], &dummy));
 }
 
+TEST(UtilTest, WideCharsLen) {
+  // "a𠮟b"
+  const string input_utf8 = "a\360\240\256\237b";
+  EXPECT_EQ(4, Util::WideCharsLen(input_utf8));
+  EXPECT_EQ(0, Util::WideCharsLen(Util::SubString(input_utf8, 0, 0)));
+  EXPECT_EQ(1, Util::WideCharsLen(Util::SubString(input_utf8, 0, 1)));
+  EXPECT_EQ(3, Util::WideCharsLen(Util::SubString(input_utf8, 0, 2)));
+  EXPECT_EQ(4, Util::WideCharsLen(Util::SubString(input_utf8, 0, 3)));
+}
+
 TEST(UtilTest, UTF8ToWide) {
   const string input_utf8 = "abc";
   wstring output_wide;
@@ -1879,8 +2193,90 @@ TEST(UtilTest, UTF8ToWide) {
   Util::WideToUTF8(output_wide, &output_utf8);
   EXPECT_EQ("abc", output_utf8);
 }
+
+TEST(UtilTest, WideToUTF8_SurrogatePairSupport) {
+  // Visual C++ 2008 does not support embedding surrogate pair in string
+  // literals like L"\uD842\uDF9F". This is why we use wchar_t array instead.
+  // "𠮟"
+  const wchar_t input_wide[] = {0xD842, 0xDF9F, 0};
+  string output_utf8;
+  Util::WideToUTF8(input_wide, &output_utf8);
+
+  wstring output_wide;
+  Util::UTF8ToWide(output_utf8, &output_wide);
+
+  EXPECT_EQ("\360\240\256\237", output_utf8);
+  EXPECT_EQ(input_wide, output_wide);
+}
 #endif  // OS_WINDOWS
 
+TEST(UtilTest, CopyFile) {
+  // just test rename operation works as intended
+  const string from = Util::JoinPath(FLAGS_test_tmpdir,
+                                     "copy_from");
+  const string to = Util::JoinPath(FLAGS_test_tmpdir,
+                                   "copy_to");
+  Util::Unlink(from);
+  Util::Unlink(to);
+
+  const char kData[] = "This is a test";
+
+  {
+    OutputFileStream ofs(from.c_str(), ios::binary);
+    ofs.write(kData, arraysize(kData));
+  }
+
+  EXPECT_TRUE(Util::CopyFile(from, to));
+  Mmap<char> mmap;
+  EXPECT_TRUE(mmap.Open(to.c_str()));
+
+  EXPECT_EQ(arraysize(kData), mmap.GetFileSize());
+  EXPECT_EQ(0, memcmp(mmap.begin(), kData, mmap.GetFileSize()));
+
+  Util::Unlink(from);
+  Util::Unlink(to);
+}
+
+TEST(UtilTest, IsEqualFile) {
+  const string filename1 = Util::JoinPath(FLAGS_test_tmpdir, "test1");
+  const string filename2 = Util::JoinPath(FLAGS_test_tmpdir, "test2");
+  Util::Unlink(filename1);
+  Util::Unlink(filename2);
+  EXPECT_FALSE(Util::IsEqualFile(filename1, filename2));
+
+  const char kTestData1[] = "test data1";
+  const char kTestData2[] = "test data2";
+
+  {
+    OutputFileStream ofs1(filename1.c_str());
+    ofs1 << kTestData1;
+  }
+  EXPECT_FALSE(Util::IsEqualFile(filename1, filename2));
+
+  {
+    OutputFileStream ofs2(filename2.c_str());
+    ofs2 << kTestData1;
+  }
+
+  EXPECT_TRUE(Util::IsEqualFile(filename1, filename2));
+
+  {
+    OutputFileStream ofs2(filename2.c_str());
+    ofs2 << kTestData1;
+    ofs2 << kTestData1;
+  }
+  EXPECT_FALSE(Util::IsEqualFile(filename1, filename2));
+
+  {
+    OutputFileStream ofs2(filename2.c_str());
+    ofs2 << kTestData2;
+  }
+  EXPECT_FALSE(Util::IsEqualFile(filename1, filename2));
+
+  Util::Unlink(filename1);
+  Util::Unlink(filename2);
+}
+
 TEST(UtilTest, AtomicRename) {
   // just test rename operation works as intended
   const string from = Util::JoinPath(FLAGS_test_tmpdir,
@@ -1966,4 +2362,370 @@ TEST(UtilTest, Fingerprint32WithSeed_uint32) {
   EXPECT_EQ(num_hash, str_hash) << num_hash << " != " << str_hash;
 }
 
+  // ArabicToWideArabic TEST
+TEST(UtilTest, ArabicToWideArabicTest) {
+  string arabic;
+  vector<Util::NumberString> output;
+
+  arabic = "12345";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToWideArabic(arabic, &output));
+  ASSERT_EQ(output.size(), 2);
+  // "１２３４５"
+  EXPECT_EQ("\xE4\xB8\x80\xE4\xBA\x8C"
+            "\xE4\xB8\x89\xE5\x9B\x9B\xE4\xBA\x94", output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_KANJI_ARABIC, output[0].style);
+  // "一二三四五"
+  EXPECT_EQ("\xEF\xBC\x91\xEF\xBC\x92"
+            "\xEF\xBC\x93\xEF\xBC\x94\xEF\xBC\x95", output[1].value);
+  EXPECT_EQ(Util::NumberString::DEFAULT_STYLE, output[1].style);
+
+  arabic = "00123";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToWideArabic(arabic, &output));
+  ASSERT_EQ(output.size(), 2);
+  // "００１２３"
+  EXPECT_EQ("\xE3\x80\x87\xE3\x80\x87"
+            "\xE4\xB8\x80\xE4\xBA\x8C\xE4\xB8\x89", output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_KANJI_ARABIC, output[0].style);
+  // "〇〇一二三"
+  EXPECT_EQ("\xEF\xBC\x90\xEF\xBC\x90"
+            "\xEF\xBC\x91\xEF\xBC\x92\xEF\xBC\x93", output[1].value);
+  EXPECT_EQ(Util::NumberString::DEFAULT_STYLE, output[1].style);
+
+  arabic = "abcde";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToWideArabic(arabic, &output));
+  EXPECT_EQ(output.size(), 0);
+
+  arabic = "012abc345";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToWideArabic(arabic, &output));
+  EXPECT_EQ(output.size(), 0);
+
+  arabic = "0.001";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToWideArabic(arabic, &output));
+  EXPECT_EQ(output.size(), 0);
+
+  arabic = "-100";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToWideArabic(arabic, &output));
+  EXPECT_EQ(output.size(), 0);
+
+  arabic = "18446744073709551616";  // UINT_MAX + 1
+  EXPECT_TRUE(Util::ArabicToWideArabic(arabic, &output));
+  // "１８４４６７４４０７３７０９５５１６１６"
+  EXPECT_EQ("\xE4\xB8\x80\xE5\x85\xAB\xE5\x9B\x9B\xE5\x9B\x9B\xE5\x85"
+            "\xAD\xE4\xB8\x83\xE5\x9B\x9B\xE5\x9B\x9B\xE3\x80\x87\xE4"
+            "\xB8\x83\xE4\xB8\x89\xE4\xB8\x83\xE3\x80\x87\xE4\xB9\x9D"
+            "\xE4\xBA\x94\xE4\xBA\x94\xE4\xB8\x80\xE5\x85\xAD\xE4\xB8"
+            "\x80\xE5\x85\xAD",
+            output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_KANJI_ARABIC, output[0].style);
+}
+
+  // ArabicToKanji TEST
+TEST(UtilTest, ArabicToKanjiTest) {
+  string arabic;
+  vector<Util::NumberString> output;
+
+  arabic = "2";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToKanji(arabic, &output));
+  ASSERT_EQ(output.size(), 2);
+  // "二"
+  EXPECT_EQ("\xE4\xBA\x8C", output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_KANJI, output[0].style);
+  // "弐"
+  EXPECT_EQ("\xE5\xBC\x90", output[1].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_OLD_KANJI, output[1].style);
+
+  arabic = "10";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToKanji(arabic, &output));
+  ASSERT_EQ(output.size(), 3);
+  // "十"
+  EXPECT_EQ("\xE5\x8D\x81", output[0].value);
+  // "壱拾"
+  EXPECT_EQ("\xE5\xA3\xB1\xE6\x8B\xBE", output[1].value);
+  // "拾"
+  EXPECT_EQ("\xE6\x8B\xBE", output[2].value);
+
+  arabic = "15";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToKanji(arabic, &output));
+  ASSERT_EQ(output.size(), 2);
+  // "十五"
+  EXPECT_EQ("\xE5\x8D\x81\xE4\xBA\x94", output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_KANJI, output[0].style);
+  // "壱拾五"
+  EXPECT_EQ("\xE5\xA3\xB1\xE6\x8B\xBE\xE4\xBA\x94", output[1].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_OLD_KANJI, output[1].style);
+
+  arabic = "20";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToKanji(arabic, &output));
+  ASSERT_EQ(output.size(), 3);
+  // "二十"
+  EXPECT_EQ("\xE4\xBA\x8C\xE5\x8D\x81", output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_KANJI, output[0].style);
+  // "弐拾"
+  EXPECT_EQ("\xE5\xBC\x90\xE6\x8B\xBE", output[1].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_OLD_KANJI, output[1].style);
+  // "廿"
+  EXPECT_EQ("\xE5\xBB\xBF", output[2].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_OLD_KANJI, output[2].style);
+
+  arabic = "25";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToKanji(arabic, &output));
+  ASSERT_EQ(output.size(), 3);
+  // "二十五"
+  EXPECT_EQ("\xE4\xBA\x8C\xE5\x8D\x81\xE4\xBA\x94", output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_KANJI, output[0].style);
+  // "弐拾五"
+  EXPECT_EQ("\xE5\xBC\x90\xE6\x8B\xBE\xE4\xBA\x94", output[1].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_OLD_KANJI, output[1].style);
+  // "廿五"
+  EXPECT_EQ("\xE5\xBB\xBF\xE4\xBA\x94", output[2].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_OLD_KANJI, output[2].style);
+
+  arabic = "12345";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToKanji(arabic, &output));
+  ASSERT_EQ(output.size(), 5);
+  // "一万二千三百四十五"
+  EXPECT_EQ("\xE4\xB8\x80\xE4\xB8\x87\xE4\xBA\x8C\xE5\x8D\x83"
+            "\xE4\xB8\x89\xE7\x99\xBE\xE5\x9B\x9B\xE5\x8D\x81"
+            "\xE4\xBA\x94", output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_KANJI, output[0].style);
+
+  // "壱万弐千参百四拾五"
+  EXPECT_EQ("\xE5\xA3\xB1\xE4\xB8\x87\xE5\xBC\x90\xE5\x8D\x83"
+            "\xE5\x8F\x82\xE7\x99\xBE\xE5\x9B\x9B\xE6\x8B\xBE"
+            "\xE4\xBA\x94", output[1].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_OLD_KANJI, output[1].style);
+
+  // "壱万弐阡参百四拾五"
+  EXPECT_EQ("\xE5\xA3\xB1\xE4\xB8\x87\xE5\xBC\x90\xE9\x98\xA1"
+            "\xE5\x8F\x82\xE7\x99\xBE\xE5\x9B\x9B\xE6\x8B\xBE"
+            "\xE4\xBA\x94", output[2].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_OLD_KANJI, output[2].style);
+
+  // "壱萬弐千参百四拾五"
+  EXPECT_EQ("\xE5\xA3\xB1\xE8\x90\xAC\xE5\xBC\x90\xE5\x8D\x83"
+            "\xE5\x8F\x82\xE7\x99\xBE\xE5\x9B\x9B\xE6\x8B\xBE"
+            "\xE4\xBA\x94", output[3].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_OLD_KANJI, output[3].style);
+
+  // "壱萬弐阡参百四拾五"
+  EXPECT_EQ("\xE5\xA3\xB1\xE8\x90\xAC\xE5\xBC\x90\xE9\x98\xA1"
+            "\xE5\x8F\x82\xE7\x99\xBE\xE5\x9B\x9B\xE6\x8B\xBE"
+            "\xE4\xBA\x94", output[4].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_OLD_KANJI, output[4].style);
+
+  arabic = "asf56789";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToKanji(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "0.001";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToKanji(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "-100";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToKanji(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "18446744073709551615";  // UINT64_MAX  + 1
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToKanji(arabic, &output));
+  // "千八百四十四京六千七百四十四兆七百三十七億九百五十五万千六百十五"
+  EXPECT_EQ("\xE5\x8D\x83\xE5\x85\xAB\xE7\x99\xBE\xE5\x9B\x9B"
+            "\xE5\x8D\x81\xE5\x9B\x9B\xE4\xBA\xAC\xE5\x85\xAD"
+            "\xE5\x8D\x83\xE4\xB8\x83\xE7\x99\xBE\xE5\x9B\x9B"
+            "\xE5\x8D\x81\xE5\x9B\x9B\xE5\x85\x86\xE4\xB8\x83"
+            "\xE7\x99\xBE\xE4\xB8\x89\xE5\x8D\x81\xE4\xB8\x83"
+            "\xE5\x84\x84\xE4\xB9\x9D\xE7\x99\xBE\xE4\xBA\x94"
+            "\xE5\x8D\x81\xE4\xBA\x94\xE4\xB8\x87\xE5\x8D\x83"
+            "\xE5\x85\xAD\xE7\x99\xBE\xE5\x8D\x81\xE4\xBA\x94",
+            output[0].value);
+}
+
+  // ArabicToSeparatedArabic TEST
+TEST(UtilTest, ArabicToSeparatedArabicTest) {
+  string arabic;
+  vector<Util::NumberString> output;
+
+  arabic = "4";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToSeparatedArabic(arabic, &output));
+  ASSERT_EQ(output.size(), 2);
+
+  EXPECT_EQ("4", output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_SEPARATED_ARABIC_HALFWIDTH,
+            output[0].style);
+  // "４"
+  EXPECT_EQ("\xEF\xBC\x94", output[1].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_SEPARATED_ARABIC_FULLWIDTH,
+            output[1].style);
+
+  arabic = "123456789";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToSeparatedArabic(arabic, &output));
+  ASSERT_EQ(output.size(), 2);
+
+  EXPECT_EQ("123,456,789", output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_SEPARATED_ARABIC_HALFWIDTH,
+            output[0].style);
+  // "１２３，４５６，７８９"
+  EXPECT_EQ("\xEF\xBC\x91\xEF\xBC\x92\xEF\xBC\x93\xEF\xBC\x8C"
+            "\xEF\xBC\x94\xEF\xBC\x95\xEF\xBC\x96\xEF\xBC\x8C"
+            "\xEF\xBC\x97\xEF\xBC\x98\xEF\xBC\x99", output[1].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_SEPARATED_ARABIC_FULLWIDTH,
+            output[1].style);
+
+  arabic = "0123456789";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToSeparatedArabic(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "asdf0123456789";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToSeparatedArabic(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "0.001";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToSeparatedArabic(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "-100";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToSeparatedArabic(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "18446744073709551616";  // UINT64_MAX + 1
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToSeparatedArabic(arabic, &output));
+  EXPECT_EQ("18,446,744,073,709,551,616", output[0].value);
+}
+
+  // ArabicToOtherForms
+TEST(UtilTest, ArabicToOtherFormsTest) {
+  string arabic;
+  vector<Util::NumberString> output;
+
+  arabic = "5";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToOtherForms(arabic, &output));
+  ASSERT_EQ(output.size(), 3);
+
+  // "Ⅴ"
+  EXPECT_EQ("\xE2\x85\xA4", output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_ROMAN_CAPITAL, output[0].style);
+
+  // "ⅴ"
+  EXPECT_EQ("\xE2\x85\xB4", output[1].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_ROMAN_SMALL, output[1].style);
+
+  // "⑤"
+  EXPECT_EQ("\xE2\x91\xA4", output[2].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_CIRCLED, output[2].style);
+
+  arabic = "0123456789";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToOtherForms(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "asdf0123456789";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToOtherForms(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "0.001";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToOtherForms(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "-100";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToOtherForms(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "18446744073709551616";  // UINT64_MAX + 1
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToOtherForms(arabic, &output));
+}
+
+  // ArabicToOtherRadixes
+TEST(UtilTest, ArabicToOtherRadixesTest) {
+  string arabic;
+  vector<Util::NumberString> output;
+
+  arabic = "1";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToOtherRadixes(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "2";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToOtherRadixes(arabic, &output));
+  ASSERT_EQ(output.size(), 1);
+
+  arabic = "8";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToOtherRadixes(arabic, &output));
+  ASSERT_EQ(output.size(), 2);
+  EXPECT_EQ("010", output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_OCT, output[0].style);
+  EXPECT_EQ("0b1000", output[1].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_BIN, output[1].style);
+
+  arabic = "16";
+  output.clear();
+  EXPECT_TRUE(Util::ArabicToOtherRadixes(arabic, &output));
+  ASSERT_EQ(output.size(), 3);
+  EXPECT_EQ("0x10", output[0].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_HEX, output[0].style);
+  EXPECT_EQ("020", output[1].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_OCT, output[1].style);
+  EXPECT_EQ("0b10000", output[2].value);
+  EXPECT_EQ(Util::NumberString::NUMBER_BIN, output[2].style);
+
+  arabic = "asdf0123456789";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToOtherRadixes(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "0.001";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToOtherRadixes(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "-100";
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToOtherRadixes(arabic, &output));
+  ASSERT_EQ(output.size(), 0);
+
+  arabic = "18446744073709551616";  // UINT64_MAX + 1
+  output.clear();
+  EXPECT_FALSE(Util::ArabicToOtherRadixes(arabic, &output));
+}
+
+
+TEST(UtilTGest, RandomSeedTest) {
+  Util::SetRandomSeed(0);
+  const int first_try = Util::Random(INT_MAX);
+  const int second_try = Util::Random(INT_MAX);
+  EXPECT_NE(first_try, second_try);
+
+  // Reset the seed.
+  Util::SetRandomSeed(0);
+  EXPECT_EQ(first_try, Util::Random(INT_MAX));
+}
+
 }  // namespace mozc
