@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -39,6 +39,7 @@
 #include "base/file_stream.h"
 #include "base/process.h"
 #include "base/run_level.h"
+#include "base/singleton.h"
 #include "base/util.h"
 #include "base/version.h"
 #include "ipc/ipc.h"
@@ -56,7 +57,7 @@ namespace mozc {
 namespace client {
 
 namespace {
-const char kServerAddress[]    = "session"; // name for the IPC connection.
+const char kServerAddress[]    = "session";  // name for the IPC connection.
 const int    kResultBufferSize = 8192 * 32;   // size of IPC buffer
 const size_t kMaxPlayBackSize  = 512;   // size of maximum history
 
@@ -536,7 +537,7 @@ bool Session::PingServer() const {
   InitInput(&input);
   input.set_type(commands::Input::NO_OPERATION);
 
- // Call IPC
+  // Call IPC
   scoped_ptr<IPCClientInterface> client(
       client_factory_->NewClient(kServerAddress,
                                  server_launcher_->server_program()));
@@ -565,6 +566,7 @@ bool Session::PingServer() const {
   return true;
 }
 
+
 bool Session::CallCommand(commands::Input::CommandType type) {
   commands::Input input;
   InitInput(&input);
@@ -799,6 +801,42 @@ bool Session::IsAbortKey(const commands::KeyEvent &key) {
         key.modifier_keys(1) == commands::KeyEvent::CTRL));
 }
 
+bool Session::TranslateProtoBufToMozcToolArg(const commands::Output &output,
+                                             string *mode) {
+  if (!output.has_launch_tool_mode() || mode == NULL) {
+    return false;
+  }
+
+  switch (output.launch_tool_mode()) {
+    case commands::Output::CONFIG_DIALOG:
+      mode->assign("config_dialog");
+      break;
+    case commands::Output::DICTIONARY_TOOL:
+      mode->assign("dictionary_tool");
+      break;
+    case commands::Output::WORD_REGISTER_DIALOG:
+      mode->assign("word_register_dialog");
+      break;
+    case commands::Output::NO_TOOL:
+    default:
+      // do nothing
+      return false;
+      break;
+  }
+
+  return true;
+}
+
+bool Session::LaunchToolWithProtoBuf(const commands::Output &output) {
+  string mode;
+  if (!TranslateProtoBufToMozcToolArg(output, &mode)) {
+    return false;
+  }
+
+  // TODO(nona): extends output message to support extra argument.
+  return LaunchTool(mode, "");
+}
+
 bool Session::LaunchTool(const string &mode, const string &extra_arg) {
   // Don't execute any child process if the parent process is not
   // in proper runlevel.
@@ -884,5 +922,28 @@ bool Session::OpenBrowser(const string &url) {
   return true;
 }
 
+namespace {
+class DefaultSessionFactory : public SessionFactoryInterface {
+ public:
+  virtual SessionInterface *NewSession() {
+    return new Session;
+  }
+};
+
+SessionFactoryInterface *g_session_factory = NULL;
+}  // namespace
+
+SessionInterface *SessionFactory::NewSession() {
+  if (g_session_factory == NULL) {
+    return Singleton<DefaultSessionFactory>::get()->NewSession();
+  } else {
+    return g_session_factory->NewSession();
+  }
+}
+
+void SessionFactory::SetSessionFactory(
+    SessionFactoryInterface *session_factory) {
+  g_session_factory = session_factory;
+}
 }  // namespace client
 }  // namespace mozc
