@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,6 +29,7 @@
 
 #include "converter/converter_interface.h"
 
+#include <climits>
 #include <string>
 #include <utility>
 #include <vector>
@@ -36,12 +37,15 @@
 #include "base/mutex.h"
 #include "base/singleton.h"
 #include "base/util.h"
+#include "composer/composer.h"
 #include "transliteration/transliteration.h"
-#include "converter/character_form_manager.h"
-#include "converter/focus_candidate_handler.h"
+#include "converter/connector_interface.h"
 #include "converter/immutable_converter_interface.h"
+#include "converter/node_allocator.h"
 #include "converter/segments.h"
 #include "dictionary/dictionary_interface.h"
+#include "dictionary/suffix_dictionary.h"
+#include "dictionary/pos_matcher.h"
 #include "prediction/predictor_interface.h"
 #include "rewriter/rewriter_interface.h"
 
@@ -57,6 +61,8 @@ class ConverterImpl : public ConverterInterface {
 
   bool StartConversion(Segments *segments,
                        const string &key) const;
+  bool StartConversionWithComposer(Segments *segments,
+                                   const composer::Composer *composer) const;
   bool StartReverseConversion(Segments *segments,
                               const string &key) const;
   bool StartPrediction(Segments *segments,
@@ -67,9 +73,6 @@ class ConverterImpl : public ConverterInterface {
   bool CancelConversion(Segments *segments) const;
   bool ResetConversion(Segments *segments) const;
   bool RevertConversion(Segments *segments) const;
-  bool GetCandidates(Segments *segments,
-                     size_t segment_index,
-                     size_t candidate_size) const;
   bool CommitSegmentValue(Segments *segments,
                           size_t segment_index,
                           int    candidate_index) const;
@@ -89,14 +92,10 @@ class ConverterImpl : public ConverterInterface {
                      const uint8 *new_size_array,
                      size_t array_size) const;
   bool Sync() const;
+  bool Reload() const;
   bool ClearUserHistory() const;
   bool ClearUserPrediction() const;
   bool ClearUnusedUserPrediction() const;
-
- private:
-  RewriterInterface *rewriter_;
-  PredictorInterface *predictor_;
-  ImmutableConverterInterface *immutable_converter_;
 };
 
 size_t GetSegmentIndex(const Segments *segments,
@@ -112,13 +111,13 @@ size_t GetSegmentIndex(const Segments *segments,
 void SetKey(Segments *segments, const string &key) {
   segments->set_max_history_segments_size(4);
   segments->clear_conversion_segments();
-  segments->clear_lattice();
   segments->clear_revert_entries();
+  segments->set_composer(NULL);
 
   mozc::Segment *seg = segments->add_segment();
   DCHECK(seg);
 
-  seg->clear();
+  seg->Clear();
   seg->set_key(key);
   seg->set_segment_type(mozc::Segment::FREE);
 
@@ -126,7 +125,6 @@ void SetKey(Segments *segments, const string &key) {
 }
 
 ConverterInterface *g_converter = NULL;
-
 }  // namespace
 
 ConverterInterface *ConverterFactory::GetConverter() {
@@ -141,93 +139,103 @@ void ConverterFactory::SetConverter(ConverterInterface *converter) {
   g_converter = converter;
 }
 
-ConverterImpl::ConverterImpl()
-    : rewriter_(RewriterFactory::GetRewriter()),
-      predictor_(PredictorFactory::GetPredictor()),
-      immutable_converter_(
-          ImmutableConverterFactory::GetImmutableConverter()) {
-  VLOG(1) << "ConverterImpl is created";
-}
-
+ConverterImpl::ConverterImpl() {}
 ConverterImpl::~ConverterImpl() {}
 
 bool ConverterImpl::StartConversion(Segments *segments,
                                     const string &key) const {
   SetKey(segments, key);
   segments->set_request_type(Segments::CONVERSION);
-  if (immutable_converter_->Convert(segments)) {
-    rewriter_->Rewrite(segments);
+  if (ImmutableConverterFactory::GetImmutableConverter()->Convert(segments)) {
+    RewriterFactory::GetRewriter()->Rewrite(segments);
     return true;
   }
   return false;
 }
-bool ConverterImpl::StartReverseConversion(Segments *segments,
-                                           const string &key) const {
-  LOG(WARNING) << "StartReverseConversion is not implemented";
+
+bool ConverterImpl::StartConversionWithComposer(
+    Segments *segments, const composer::Composer *composer) const {
+  string conversion_key;
+  if (composer != NULL) {
+    composer->GetQueryForConversion(&conversion_key);
+  }
+  SetKey(segments, conversion_key);
+  segments->set_composer(composer);
+  segments->set_request_type(Segments::CONVERSION);
+  if (ImmutableConverterFactory::GetImmutableConverter()->Convert(segments)) {
+    RewriterFactory::GetRewriter()->Rewrite(segments);
+    return true;
+  }
   return false;
 }
+
+bool ConverterImpl::StartReverseConversion(Segments *segments,
+                                           const string &key) const {
+  segments->Clear();
+  SetKey(segments, key);
+  segments->set_request_type(Segments::REVERSE_CONVERSION);
+  if (!ImmutableConverterFactory::GetImmutableConverter()->Convert(segments)) {
+    return false;
+  }
+  if (segments->segments_size() == 0) {
+    LOG(WARNING) << "no segments from reverse conversion";
+    return false;
+  }
+  for (int i = 0; i < segments->segments_size(); ++i) {
+    const mozc::Segment &seg = segments->segment(i);
+    if (seg.candidates_size() == 0 ||
+        seg.candidate(0).value.empty()) {
+      segments->Clear();
+      LOG(WARNING) << "got an empty segment from reverse conversion";
+      return false;
+    }
+  }
+  return true;
+}
+
 bool ConverterImpl::StartPrediction(Segments *segments,
                                     const string &key) const {
   SetKey(segments, key);
   segments->set_request_type(Segments::PREDICTION);
-  return predictor_->Predict(segments);
+  if (PredictorFactory::GetPredictor()->Predict(segments)) {
+    RewriterFactory::GetRewriter()->Rewrite(segments);
+    return true;
+  }
+  return false;
 }
 
 bool ConverterImpl::StartSuggestion(Segments *segments,
                                     const string &key) const {
   SetKey(segments, key);
   segments->set_request_type(Segments::SUGGESTION);
-  return predictor_->Predict(segments);
+  if (PredictorFactory::GetPredictor()->Predict(segments)) {
+    RewriterFactory::GetRewriter()->Rewrite(segments);
+    return true;
+  }
+  return false;
 }
 
 bool ConverterImpl::FinishConversion(Segments *segments) const {
   // revert SUBMITTED segments to FIXED_VALUE
   for (int i = 0; i < segments->segments_size(); ++i) {
     Segment *seg = segments->mutable_segment(i);
+    DCHECK(seg);
     if (seg->segment_type() == Segment::SUBMITTED) {
       seg->set_segment_type(Segment::FIXED_VALUE);
     }
-  }
-
-  // save character form
-  for (int i = 0; i < segments->conversion_segments_size(); ++i) {
-    const Segment &seg = segments->conversion_segment(i);
-    if (seg.candidates_size() == 0) {
-      continue;
-    }
-
-    const Segment::Candidate &cand = seg.candidate(0);
-    if (!cand.can_expand_alternative) {
-      continue;
-    }
-
-    switch (cand.style) {
-      case Segment::Candidate::NUMBER_SEPARATED_ARABIC_HALFWIDTH:
-        // treat NUMBER_SEPARATED_ARABIC as half_width num
-        CharacterFormManager::GetCharacterFormManager()->
-            SetCharacterForm("0", config::Config::HALF_WIDTH);
-        break;
-      case Segment::Candidate::NUMBER_SEPARATED_ARABIC_FULLWIDTH:
-        // treat NUMBER_SEPARATED_WIDE_ARABIC as full_width num
-        CharacterFormManager::GetCharacterFormManager()->
-            SetCharacterForm("0", config::Config::FULL_WIDTH);
-        break;
-      default:
-        CharacterFormManager::GetCharacterFormManager()->
-            GuessAndSetCharacterForm(cand.value);
-        break;
+    if (seg->candidates_size() > 0) {
+      ConverterUtil::CompletePOSIds(seg->mutable_candidate(0));
     }
   }
 
   segments->clear_revert_entries();
   if (segments->request_type() == Segments::CONVERSION) {
-    rewriter_->Finish(segments);
+    RewriterFactory::GetRewriter()->Finish(segments);
   }
-  predictor_->Finish(segments);
+  PredictorFactory::GetPredictor()->Finish(segments);
 
-  segments->clear_lattice();
-
-  const int start_index = max(0, static_cast<int>(
+  const int start_index = max(0,
+                              static_cast<int>(
                                   segments->segments_size() -
                                   segments->max_history_segments_size()));
   for (int i = 0; i < start_index; ++i) {
@@ -236,6 +244,7 @@ bool ConverterImpl::FinishConversion(Segments *segments) const {
 
   for (size_t i = 0; i < segments->segments_size(); ++i) {
     Segment *seg = segments->mutable_segment(i);
+    DCHECK(seg);
     seg->set_segment_type(Segment::HISTORY);
   }
 
@@ -243,7 +252,6 @@ bool ConverterImpl::FinishConversion(Segments *segments) const {
 }
 
 bool ConverterImpl::CancelConversion(Segments *segments) const {
-  segments->clear_lattice();
   segments->clear_conversion_segments();
   return true;
 }
@@ -257,29 +265,11 @@ bool ConverterImpl::RevertConversion(Segments *segments) const {
   if (segments->revert_entries_size() == 0) {
     return true;
   }
-  predictor_->Revert(segments);
+  PredictorFactory::GetPredictor()->Revert(segments);
   segments->clear_revert_entries();
   return true;
 }
 
-bool ConverterImpl::GetCandidates(Segments *segments,
-                                  size_t segment_index,
-                                  size_t candidate_size) const {
-  if (segments->request_type() != Segments::CONVERSION) {
-    return true;
-  }
-
-  segment_index = GetSegmentIndex(segments, segment_index);
-  if (segment_index == kErrorIndex) {
-    return false;
-  }
-
-  Segment *segment = segments->mutable_segment(segment_index);
-  segment->GetCandidates(candidate_size);
-
-  return true;
-}
-
 bool ConverterImpl::CommitSegmentValue(Segments *segments,
                                        size_t segment_index,
                                        int candidate_index) const {
@@ -299,7 +289,7 @@ bool ConverterImpl::CommitSegmentValue(Segments *segments,
   segment->move_candidate(candidate_index, 0);
 
   if (candidate_index != 0) {
-    segment->mutable_candidate(0)->learning_type
+    segment->mutable_candidate(0)->attributes
         |= Segment::Candidate::RERANKED;
   }
 
@@ -314,12 +304,10 @@ bool ConverterImpl::FocusSegmentValue(Segments *segments,
     return false;
   }
 
-  return FocusCandidateHandler::FocusSegmentValue(segments,
-                                                  segment_index,
-                                                  candidate_index);
+  return RewriterFactory::GetRewriter()->Focus(segments,
+                                               segment_index, candidate_index);
 }
 
-
 bool ConverterImpl::FreeSegmentValue(Segments *segments,
                                      size_t segment_index) const {
   segment_index = GetSegmentIndex(segments, segment_index);
@@ -334,7 +322,7 @@ bool ConverterImpl::FreeSegmentValue(Segments *segments,
     return false;
   }
 
-  return immutable_converter_->Convert(segments);
+  return ImmutableConverterFactory::GetImmutableConverter()->Convert(segments);
 }
 
 bool ConverterImpl::SubmitFirstSegment(Segments *segments,
@@ -401,7 +389,7 @@ bool ConverterImpl::ResizeSegment(Segments *segments,
     }
 
     Segment *segment = segments->mutable_segment(segment_index);
-    segment->clear();
+    segment->Clear();
     segment->set_segment_type(Segment::FIXED_BOUNDARY);
     segment->set_key(new_key);
 
@@ -417,7 +405,7 @@ bool ConverterImpl::ResizeSegment(Segments *segments,
   } else if (offset_length < 0) {
     if (cur_length + offset_length > 0) {
       Segment *segment1 = segments->mutable_segment(segment_index);
-      segment1->clear();
+      segment1->Clear();
       segment1->set_segment_type(Segment::FIXED_BOUNDARY);
       string new_key;
       Util::SubString(cur_segment_key,
@@ -451,11 +439,11 @@ bool ConverterImpl::ResizeSegment(Segments *segments,
 
   segments->set_resized(true);
 
-  if (!immutable_converter_->Convert(segments)) {
+  if (!ImmutableConverterFactory::GetImmutableConverter()->Convert(segments)) {
     return false;
   }
 
-  rewriter_->Rewrite(segments);
+  RewriterFactory::GetRewriter()->Rewrite(segments);
 
   return true;
 }
@@ -513,36 +501,39 @@ bool ConverterImpl::ResizeSegment(Segments *segments,
 
   segments->set_resized(true);
 
-  if (!immutable_converter_->Convert(segments)) {
+  if (!ImmutableConverterFactory::GetImmutableConverter()->Convert(segments)) {
     return false;
   }
 
-  rewriter_->Rewrite(segments);
+  RewriterFactory::GetRewriter()->Rewrite(segments);
 
   return true;
 }
 
 bool ConverterImpl::Sync() const {
-  return (rewriter_->Sync() && predictor_->Sync());
+  return (RewriterFactory::GetRewriter()->Sync() &&
+          PredictorFactory::GetPredictor()->Sync() &&
+          DictionaryFactory::GetDictionary()->Sync());
+}
+
+bool ConverterImpl::Reload() const {
+  return (RewriterFactory::GetRewriter()->Reload() &&
+          PredictorFactory::GetPredictor()->Reload() &&
+          DictionaryFactory::GetDictionary()->Reload());
 }
 
 bool ConverterImpl::ClearUserHistory() const {
-  rewriter_->Clear();
-
-  // TODO(taku): It's not the best timing to clear the character form
-  // manager's data.
-  CharacterFormManager::GetCharacterFormManager()->ClearHistory();
-
+  RewriterFactory::GetRewriter()->Clear();
   return true;
 }
 
 bool ConverterImpl::ClearUserPrediction() const {
-  predictor_->ClearAllHistory();
+  PredictorFactory::GetPredictor()->ClearAllHistory();
   return true;
 }
 
 bool ConverterImpl::ClearUnusedUserPrediction() const {
-  predictor_->ClearUnusedHistory();
+  PredictorFactory::GetPredictor()->ClearUnusedHistory();
   return true;
 }
 
@@ -555,7 +546,7 @@ void ConverterUtil::InitSegmentsFromString(const string &key,
   // CONVERSION from SUGGESTION now.
   segments->set_request_type(Segments::CONVERSION);
   Segment *segment = segments->add_segment();
-  segment->clear();
+  segment->Clear();
   segment->set_key(key);
   segment->set_segment_type(Segment::FIXED_VALUE);
   Segment::Candidate *c = segment->add_candidate();
@@ -565,4 +556,79 @@ void ConverterUtil::InitSegmentsFromString(const string &key,
   c->key = key;
   c->content_key = key;
 }
+
+// static
+void ConverterUtil::CompletePOSIds(Segment::Candidate *candidate) {
+  DCHECK(candidate);
+  if (candidate->value.empty()) {
+    return;
+  }
+
+  if (candidate->lid == 0) {
+    candidate->lid = POSMatcher::GetUnknownId();
+    VLOG(1) << "Set LID: " << candidate->lid;
+  }
+
+  if (candidate->rid == 0) {
+    const ConnectorInterface *connector =
+        ConnectorFactory::GetConnector();
+    const DictionaryInterface *dictionary =
+        SuffixDictionaryFactory::GetSuffixDictionary();
+
+    DCHECK(connector);
+    DCHECK(dictionary);
+
+    const string &value = candidate->value;
+
+    NodeAllocator allocator;
+    Node *node = dictionary->LookupPredictive("", 0, &allocator);
+    DCHECK(node);
+
+    const Node *best_node = NULL;
+    int best_cost = INT_MAX;
+
+    // Find a entry which is the suffix of |value|.
+    // Suffix dictionary supposes to have frequent suffix patterns,
+    // like particle, auxiliary verbs and functional verbs.
+    // Since hiragana-sequence which were not passed to the converter tend
+    // to be functional expressions, it is reasonable to use suffix
+    // dictionary to guess POS ids.
+    for (; node != NULL; node = node->bnext) {
+      if (!Util::EndsWith(value, node->value)) {
+        continue;
+      }
+
+      // Preserve the longest suffix match.
+      if (best_node == NULL ||
+          best_node->value.size() < node->value.size()) {
+        best_node = node;
+        best_cost = connector->GetTransitionCost(0, node->lid) +
+            node->wcost;
+        continue;
+      }
+
+      // If there are multiple nodes which have the
+      // same suffix length, select one who has the minimum
+      // language model cost.
+      if (best_node->value.size() == node->value.size()) {
+        const int cost = connector->GetTransitionCost(0, node->lid) +
+            node->wcost;
+        if (cost < best_cost) {
+          best_cost = cost;
+          best_node = node;
+        }
+      }
+    }
+
+    if (best_node != NULL) {
+      candidate->rid = best_node->rid;
+    } else {
+      // Cannot find suffix match. Use UnkownId just in case.
+      // TODO(team): we might want to call reverse conversion.
+      candidate->rid = POSMatcher::GetUnknownId();
+    }
+
+    VLOG(1) << "Set RID: " << candidate->rid << " " << value;
+  }
+}
 }  // namespace mozc
