@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,10 +32,13 @@
 
 #include <string>
 #include "base/base.h"
+#include "converter/segments.h"
 
 namespace mozc {
 
-class Segments;
+namespace composer {
+  class Composer;
+}  // namespace composer
 
 class ConverterInterface {
  public:
@@ -44,6 +47,10 @@ class ConverterInterface {
   virtual bool StartConversion(Segments *segments,
                                const string &key) const = 0;
 
+  // Start conversion with composer.
+  virtual bool StartConversionWithComposer(
+      Segments *segments, const composer::Composer *composer) const = 0;
+
   // Start reverse conversion with key.
   virtual bool StartReverseConversion(Segments *segments,
                                       const string &key) const = 0;
@@ -70,9 +77,13 @@ class ConverterInterface {
   virtual bool RevertConversion(Segments *segments) const = 0;
 
   // Expand the bunsetsu-segment at "segment_index" by candidate_size
+  // DEPRECATED: This method doesn't take any effect.
+  // TODO(taku): remove this method.
   virtual bool GetCandidates(Segments *segments,
                              size_t segment_index,
-                             size_t candidate_size) const = 0;
+                             size_t candidate_size) const {
+    return true;
+  }
 
   // Commit candidate
   virtual bool CommitSegmentValue(Segments *segments,
@@ -114,9 +125,14 @@ class ConverterInterface {
                              const uint8 *new_size_array,
                              size_t array_size) const = 0;
 
-  // Sync user history data to local file system.
+  // Sync mutable user data to local file system.
   virtual bool Sync() const = 0;
 
+  // Reload mutable data from local file system.
+  virtual bool Reload() const = 0;
+
+  // TODO(taku): Remove below methods as they are too specific.
+
   // clear user history data
   virtual bool ClearUserHistory() const = 0;
 
@@ -159,6 +175,14 @@ class ConverterUtil {
   static void InitSegmentsFromString(const string &key,
                                      const string &preedit,
                                      Segments *segments);
+
+  // Complete Left id/Right id if they are not defined.
+  // Some users don't push conversion button but directly
+  // input hiragana sequence only with composition mode. Converter
+  // cannot know which POS ids should be used for these directly-
+  // input strings. This function estimates IDs from value heuristically.
+  static void CompletePOSIds(Segment::Candidate *candidate);
+
  private:
   ConverterUtil() {}
   ~ConverterUtil() {}
