@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -43,6 +43,8 @@ namespace {
 
 const char kConversionExpect[]    = "Conversion Expect";
 const char kConversionNotExpect[] = "Conversion Not Expect";
+const char kReverseConversionExpect[]    = "ReverseConversion Expect";
+const char kReverseConversionNotExpect[] = "ReverseConversion Not Expect";
 const char kPredictionExpect[]    = "Prediction Expect";
 const char kPredictionNotExpect[] = "Prediction Not Expect";
 
@@ -98,14 +100,12 @@ bool QualityRegressionUtil::ConvertAndTest(const string &key,
   converter_->ResetConversion(segments_.get());
   actual_value->clear();
 
-  const size_t expand_size = expected_rank + 200;
-
   if (command == kConversionExpect ||
       command == kConversionNotExpect) {
     converter_->StartConversion(segments_.get(), key);
-    if (expected_rank > 0) {
-      converter_->GetCandidates(segments_.get(), 0, expand_size);
-    }
+  } else if (command == kReverseConversionExpect ||
+    command == kReverseConversionNotExpect) {
+    converter_->StartReverseConversion(segments_.get(), key);
   } else if (command == kPredictionExpect ||
              command == kPredictionNotExpect) {
     converter_->StartPrediction(segments_.get(), key);
@@ -131,6 +131,7 @@ bool QualityRegressionUtil::ConvertAndTest(const string &key,
   bool result = (actual_rank >= 0 && actual_rank <= expected_rank);
 
   if (command == kConversionNotExpect ||
+      command == kReverseConversionNotExpect ||
       command == kPredictionNotExpect) {
     result = !result;
   }
