@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -102,11 +102,6 @@ Node *SystemDictionary::LookupPredictive(const char *str, int size,
   return LookupInternal(str, size, allocator, true, NULL);
 }
 
-Node *SystemDictionary::LookupExact(const char *str, int size,
-                                    NodeAllocatorInterface *allocator) const {
-  return NULL;
-}
-
 Node *SystemDictionary::LookupPrefix(const char *str, int size,
                                      NodeAllocatorInterface *allocator) const {
   // Not a predictive lookup. Gets all tokens match the key.
@@ -177,7 +172,14 @@ Node *SystemDictionary::CopyTokenToNode(NodeAllocatorInterface *allocator,
     // for test
     new_node = new Node();
   }
-  new_node->lid = token->lid;
+
+  if (token->lid >= kSpellingCorrectionPosOffset) {
+    new_node->lid = token->lid - kSpellingCorrectionPosOffset;
+    new_node->attributes |= Node::SPELLING_CORRECTION;
+  } else {
+    new_node->lid = token->lid;
+    new_node->attributes &= ~Node::SPELLING_CORRECTION;
+  }
   new_node->rid = token->rid;
   new_node->wcost = token->cost;
   new_node->key = token->key;
