@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,11 +32,14 @@
 #include <algorithm>
 #include <set>
 #include <string>
+
 #include "base/base.h"
 #include "base/config_file_stream.h"
 #include "base/mutex.h"
 #include "base/singleton.h"
 #include "converter/node.h"
+#include "dictionary/pos_matcher.h"
+#include "dictionary/suppression_dictionary.h"
 #include "dictionary/user_dictionary_storage.h"
 #include "dictionary/user_dictionary_util.h"
 #include "session/config.pb.h"
@@ -50,7 +53,7 @@ void ReloadUserDictionary() {
   VLOG(1) << "Reloading user dictionary";
   UserDictionary::GetUserDictionary()->AsyncReload();
   // Sync version:
-  // UserDictionary::GetUserDictionary()->Reload();
+  // UserDictionary::GetUserDictionary()->SyncReload();
 }
 
 // ReloadUserDictionary() is called by Session::Reload()
@@ -152,6 +155,10 @@ Node *UserDictionary::LookupPredictive(const char *str, int size,
     return NULL;
   }
 
+  if (GET_CONFIG(incognito_mode)) {
+    return NULL;
+  }
+
   if (!CheckReloaderAndDelete()) {
     LOG(WARNING) << "Reloader is running";
     return NULL;
@@ -168,18 +175,25 @@ Node *UserDictionary::LookupPredictive(const char *str, int size,
       lower_bound(tokens_.begin(), tokens_.end(), &key_token, POSTokenLess());
 
   for (; it != tokens_.end(); ++it) {
-    if ((*it)->key.find(key) != 0) {
+    if (!Util::StartsWith((*it)->key, key)) {
       break;
     }
 
     Node *new_node = allocator->NewNode();
-    new_node->lid = (*it)->id;
-    new_node->rid = (*it)->id;
+    DCHECK(new_node);
+    if (POSMatcher::IsSuggestOnlyWord((*it)->id)) {
+      new_node->lid = POSMatcher::GetUnknownId();
+      new_node->rid = POSMatcher::GetUnknownId();
+    } else {
+      new_node->lid = (*it)->id;
+      new_node->rid = (*it)->id;
+    }
     new_node->wcost = (*it)->cost;
     new_node->key = (*it)->key;
     new_node->value = (*it)->value;
     new_node->node_type = Node::NOR_NODE;
-    new_node->normalization_type = Node::NO_NORMALIZATION;
+    new_node->attributes |= Node::NO_VARIANTS_EXPANSION;
+    new_node->attributes |= Node::USER_DICTIONARY;
     new_node->bnext = result_node;
     result_node = new_node;
   }
@@ -197,6 +211,10 @@ Node *UserDictionary::LookupPrefix(const char *str, int size,
     return NULL;
   }
 
+  if (GET_CONFIG(incognito_mode)) {
+    return NULL;
+  }
+
   if (!CheckReloaderAndDelete()) {
     LOG(WARNING) << "Reloader is running";
     return NULL;
@@ -212,22 +230,30 @@ Node *UserDictionary::LookupPrefix(const char *str, int size,
   vector<UserPOS::Token *>::const_iterator it =
       lower_bound(tokens_.begin(), tokens_.end(), &key_token, POSTokenLess());
 
+
   for (; it != tokens_.end(); ++it) {
     if ((*it)->key > key) {
       break;
     }
-    if (key.find((*it)->key) != 0) {
+
+    if (POSMatcher::IsSuggestOnlyWord((*it)->id)) {
+      continue;
+    }
+
+    if (!Util::StartsWith(key, (*it)->key)) {
       continue;
     }
 
     Node *new_node = allocator->NewNode();
+    DCHECK(new_node);
     new_node->lid = (*it)->id;
     new_node->rid = (*it)->id;
     new_node->wcost = (*it)->cost;
     new_node->key = (*it)->key;
     new_node->value = (*it)->value;
     new_node->node_type = Node::NOR_NODE;
-    new_node->normalization_type = Node::NO_NORMALIZATION;
+    new_node->attributes |= Node::NO_VARIANTS_EXPANSION;
+    new_node->attributes |= Node::USER_DICTIONARY;
     new_node->bnext = result_node;
     result_node = new_node;
   }
@@ -242,15 +268,19 @@ Node *UserDictionary::LookupReverse(const char *str, int size,
     return NULL;
   }
 
+  if (GET_CONFIG(incognito_mode)) {
+    return NULL;
+  }
+
   return NULL;
 }
 
 bool UserDictionary::Reload() {
-  Clear();
+  return AsyncReload();
+}
 
-  if (GET_CONFIG(incognito_mode)) {
-    return true;
-  }
+bool UserDictionary::SyncReload() {
+  Clear();
 
   scoped_ptr<UserDictionaryStorage>
       storage(
@@ -261,19 +291,20 @@ bool UserDictionary::Reload() {
     return false;
   }
 
+  SuppressionDictionary::GetSuppressionDictionary()->Lock();
+
   return Load(*(storage.get()));
 }
 
 bool UserDictionary::AsyncReload() {
-  if (GET_CONFIG(incognito_mode)) {
-    return true;
-  }
-
   // now loading
   if (!CheckReloaderAndDelete()) {
     return true;
   }
 
+  SuppressionDictionary::GetSuppressionDictionary()->Lock();
+  DCHECK(SuppressionDictionary::GetSuppressionDictionary()->IsLocked());
+
   reloader_.reset(new UserDictionaryReloader(this));
   reloader_->Start();
 
@@ -293,6 +324,14 @@ bool UserDictionary::Load(const UserDictionaryStorage &storage) {
   set<uint64> seen;
   vector<UserPOS::Token> tokens;
 
+  SuppressionDictionary *suppression_dictionary =
+      SuppressionDictionary::GetSuppressionDictionary();
+  DCHECK(suppression_dictionary);
+  if (!suppression_dictionary->IsLocked()) {
+    LOG(ERROR) << "SuppressionDictionary must be locked first";
+  }
+  suppression_dictionary->Clear();
+
   for (size_t i = 0; i < storage.dictionaries_size(); ++i) {
     const UserDictionaryStorage::UserDictionary &dic =
         storage.dictionaries(i);
@@ -303,6 +342,7 @@ bool UserDictionary::Load(const UserDictionaryStorage &storage) {
     for (size_t j = 0; j < dic.entries_size(); ++j) {
       const UserDictionaryStorage::UserDictionaryEntry &entry =
           dic.entries(j);
+
       if (!UserDictionaryUtil::IsValidEntry(entry)) {
         continue;
       }
@@ -325,17 +365,24 @@ bool UserDictionary::Load(const UserDictionaryStorage &storage) {
         continue;
       }
 
-      tokens.clear();
-      UserPOS::GetTokens(reading, entry.value(), entry.pos(),
-                         UserPOS::DEFAULT, &tokens);
-      for (size_t k = 0; k < tokens.size(); ++k) {
-        tokens_.push_back(new UserPOS::Token(tokens[k]));
+      // "抑制単語"
+      if (entry.pos() == "\xE6\x8A\x91\xE5\x88\xB6\xE5\x8D\x98\xE8\xAA\x9E") {
+        suppression_dictionary->AddEntry(reading, entry.value());
+      } else {
+        tokens.clear();
+        UserPOS::GetTokens(reading, entry.value(), entry.pos(),
+                           &tokens);
+        for (size_t k = 0; k < tokens.size(); ++k) {
+          tokens_.push_back(new UserPOS::Token(tokens[k]));
+        }
       }
     }
   }
 
   sort(tokens_.begin(), tokens_.end(), POSTokenLess());
 
+  suppression_dictionary->UnLock();
+
   VLOG(1) << tokens_.size() << " user dic entries loaded";
 
   usage_stats::UsageStats::SetInteger("UserRegisteredWord",
