@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -40,9 +40,12 @@
 #include "base/file_stream.h"
 #include "base/util.h"
 #include "converter/node.h"
+#include "dictionary/suppression_dictionary.h"
 #include "dictionary/user_dictionary_storage.h"
 #include "dictionary/user_dictionary_util.h"
 #include "dictionary/user_pos.h"
+#include "session/config_handler.h"
+#include "session/config.pb.h"
 #include "testing/base/public/googletest.h"
 #include "testing/base/public/gunit.h"
 
@@ -137,7 +140,6 @@ class UserPOSMock : public UserPOS::UserPOSInterface {
   virtual bool GetTokens(const string &key,
                          const string &value,
                          const string &pos,
-                         UserPOS::CostType cost_type,
                          vector<UserPOS::Token> *tokens) const {
     if (key.empty() ||
         value.empty() ||
@@ -186,12 +188,14 @@ string GenRandomAlphabet(int size) {
 
 class UserDictionaryTest : public testing::Test {
  protected:
-  static void SetUpTestCase() {
+  virtual void SetUp() {
+    pos_mock_.reset(new UserPOSMock);
     Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
-    UserPOS::SetUserPOSInterface(new UserPOSMock);
+    UserPOS::SetUserPOSInterface(pos_mock_.get());
   }
 
-  static void TearDownTestCase() {
+  virtual void TearDown() {
+    pos_mock_.reset();
     UserPOS::SetUserPOSInterface(NULL);
   }
 
@@ -293,6 +297,8 @@ class UserDictionaryTest : public testing::Test {
       entry->set_pos(fields[2]);
     }
   }
+
+  scoped_ptr<UserPOSMock> pos_mock_;
 };
 
 TEST_F(UserDictionaryTest, TestLookupPredictive) {
@@ -413,8 +419,37 @@ TEST_F(UserDictionaryTest, TestLookupPrefix) {
   TestLookupPrefixHelper(NULL, 0, "starting", 8, *dic.get());
 }
 
+TEST_F(UserDictionaryTest, IncognitoModeTest) {
+  config::Config config;
+  config::ConfigHandler::GetConfig(&config);
+  config.set_incognito_mode(true);
+  config::ConfigHandler::SetConfig(config);
+
+  scoped_ptr<UserDictionary> dic(CreateDictionary());
+  // Wait for async reload called from the constructor.
+  dic->WaitForReloader();
+
+  {
+    UserDictionaryStorage storage("");
+    UserDictionaryTest::LoadFromString(kUserDictionary0, &storage);
+    dic->Load(storage);
+  }
+
+  TestNodeAllocator allocator;
+
+  EXPECT_EQ(NULL, dic->LookupPrefix("start", 4, &allocator));
+  EXPECT_EQ(NULL, dic->LookupPredictive("s", 1, &allocator));
+
+  config.set_incognito_mode(false);
+  config::ConfigHandler::SetConfig(config);
+
+  EXPECT_FALSE(NULL == dic->LookupPrefix("start", 4, &allocator));
+  EXPECT_FALSE(NULL == dic->LookupPredictive("s", 1, &allocator));
+}
+
 TEST_F(UserDictionaryTest, AsyncLoadTest) {
-  const string filename = Util::JoinPath(FLAGS_test_tmpdir, "test.db");
+  const string filename = Util::JoinPath(FLAGS_test_tmpdir,
+                                         "async_load_test.db");
   Util::Unlink(filename);
 
   // Create dictionary
@@ -439,6 +474,7 @@ TEST_F(UserDictionaryTest, AsyncLoadTest) {
       keys.push_back(entry->key());
     }
     EXPECT_TRUE(storage.Save());
+    EXPECT_TRUE(storage.UnLock());
   }
 
   {
@@ -456,7 +492,150 @@ TEST_F(UserDictionaryTest, AsyncLoadTest) {
                           keys[i].size(), &allocator);
       }
     }
+    dic->WaitForReloader();
   }
+  Util::Unlink(filename);
+}
+
+TEST_F(UserDictionaryTest, TestSuppressionDictionary) {
+  SuppressionDictionary *suppression_dictionary =
+      SuppressionDictionary::GetSuppressionDictionary();
+
+  scoped_ptr<UserDictionary> user_dic(CreateDictionary());
+  user_dic->WaitForReloader();
+
+  const string filename = Util::JoinPath(FLAGS_test_tmpdir,
+                                         "suppression_test.db");
+  Util::Unlink(filename);
+
+  UserDictionaryStorage storage(filename);
+
+  // Create dictionary
+  {
+    uint64 id = 0;
+    EXPECT_TRUE(storage.CreateDictionary("test", &id));
+    UserDictionaryStorage::UserDictionary *dic =
+        storage.mutable_dictionaries(0);
+    for (size_t j = 0; j < 10000; ++j) {
+      UserDictionaryStorage::UserDictionaryEntry *entry =
+          dic->add_entries();
+      entry->set_key("no_suppress_key" + Util::SimpleItoa(j));
+      entry->set_value("no_suppress_value" + Util::SimpleItoa(j));
+      entry->set_pos("noun");
+    }
+
+    for (size_t j = 0; j < 10; ++j) {
+      UserDictionaryStorage::UserDictionaryEntry *entry =
+          dic->add_entries();
+      entry->set_key("suppress_key" + Util::SimpleItoa(j));
+      entry->set_value("suppress_value" + Util::SimpleItoa(j));
+      // entry->set_pos("抑制単語");
+      entry->set_pos("\xE6\x8A\x91\xE5\x88\xB6\xE5\x8D\x98\xE8\xAA\x9E");
+    }
+
+    suppression_dictionary->Lock();
+    EXPECT_TRUE(suppression_dictionary->IsLocked());
+    user_dic->Load(storage);
+    EXPECT_FALSE(suppression_dictionary->IsLocked());
+
+    for (size_t j = 0; j < 10; ++j) {
+      EXPECT_TRUE(suppression_dictionary->SuppressEntry(
+          "suppress_key" + Util::SimpleItoa(j),
+          "suppress_value" + Util::SimpleItoa(j)));
+    }
+  }
+
+  // Remove suppression entry
+  {
+    storage.Clear();
+    uint64 id = 0;
+    EXPECT_TRUE(storage.CreateDictionary("test", &id));
+    UserDictionaryStorage::UserDictionary *dic =
+        storage.mutable_dictionaries(0);
+    for (size_t j = 0; j < 10000; ++j) {
+      UserDictionaryStorage::UserDictionaryEntry *entry =
+          dic->add_entries();
+      entry->set_key("no_suppress_key" + Util::SimpleItoa(j));
+      entry->set_value("no_suppress_value" + Util::SimpleItoa(j));
+      entry->set_pos("noun");
+    }
+
+    suppression_dictionary->Lock();
+    user_dic->Load(storage);
+    EXPECT_FALSE(suppression_dictionary->IsLocked());
+
+    for (size_t j = 0; j < 10; ++j) {
+      EXPECT_FALSE(suppression_dictionary->SuppressEntry(
+          "suppress_key" + Util::SimpleItoa(j),
+          "suppress_value" + Util::SimpleItoa(j)));
+    }
+  }
+  Util::Unlink(filename);
+}
+
+TEST_F(UserDictionaryTest, TestSuggestionOnlyWord) {
+  scoped_ptr<UserDictionary> user_dic(CreateDictionary());
+  user_dic->WaitForReloader();
+  UserPOS::SetUserPOSInterface(NULL);
+
+  const string filename = Util::JoinPath(FLAGS_test_tmpdir,
+                                         "suggestion_only_test.db");
+  Util::Unlink(filename);
+
+  UserDictionaryStorage storage(filename);
+
+  // Create dictionary
+  {
+    uint64 id = 0;
+    EXPECT_TRUE(storage.CreateDictionary("test", &id));
+    UserDictionaryStorage::UserDictionary *dic =
+        storage.mutable_dictionaries(0);
+
+    for (size_t j = 0; j < 10; ++j) {
+      UserDictionaryStorage::UserDictionaryEntry *entry =
+          dic->add_entries();
+      entry->set_key("key" + Util::SimpleItoa(j));
+      entry->set_value("default");
+      // "名詞"
+      entry->set_pos("\xE5\x90\x8D\xE8\xA9\x9E");
+    }
+
+    for (size_t j = 0; j < 10; ++j) {
+      UserDictionaryStorage::UserDictionaryEntry *entry =
+          dic->add_entries();
+      entry->set_key("key" + Util::SimpleItoa(j));
+      entry->set_value("suggest_only");
+      // "サジェストのみ"
+      entry->set_pos("\xE3\x82\xB5\xE3\x82\xB8\xE3\x82\xA7"
+                     "\xE3\x82\xB9\xE3\x83\x88\xE3\x81\xAE\xE3\x81\xBF");
+    }
+
+    user_dic->Load(storage);
+  }
+
+  TestNodeAllocator allocator;
+
+  {
+    const char kKey[] = "key0123";
+    const Node *node = user_dic->LookupPrefix(kKey, strlen(kKey),
+                                              &allocator);
+    CHECK(node);
+    for (; node != NULL; node = node->bnext) {
+      EXPECT_TRUE("suggest_only" != node->value && "default" == node->value);
+    }
+  }
+
+  {
+    const char kKey[] = "key";
+    const Node *node = user_dic->LookupPredictive(kKey, strlen(kKey),
+                                                  &allocator);
+    CHECK(node);
+    for (; node != NULL; node = node->bnext) {
+      EXPECT_TRUE("suggest_only" == node->value || "default" == node->value);
+    }
+  }
+
+  Util::Unlink(filename);
 }
 }  // namespace
 }  // namespace mozc
