@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -31,7 +31,6 @@
 
 #include <QtCore/QTimer>
 #include <QtGui/QtGui>
-#include <QtGui/QApplication>
 #include <QtGui/QProgressDialog>
 
 #include <algorithm>
@@ -226,6 +225,7 @@ class MultiByteTextLineIterator
 
   void Reset() {
     ifs_->seekg(0, ios_base::beg);
+    first_line_ = true;
   }
 
  private:
@@ -286,7 +286,8 @@ DictionaryTool::DictionaryTool(QWidget *parent)
       new_action_(NULL), rename_action_(NULL), delete_action_(NULL),
       import_create_action_(NULL), import_append_action_(NULL),
       export_action_(NULL), import_default_ime_action_(NULL),
-      session_(new client::Session) {
+      session_(new client::Session),
+      is_available_(true) {
   setupUi(this);
 
   session_->set_timeout(kSessionTimeout);
@@ -299,7 +300,8 @@ DictionaryTool::DictionaryTool(QWidget *parent)
     QMessageBox::information(
         this, window_title_,
         tr("Another process is accessing the user dictionary file."));
-    QApplication::quit();
+    is_available_ = false;
+    return;
   }
 
   // main window
@@ -467,9 +469,9 @@ DictionaryTool::DictionaryTool(QWidget *parent)
                       << static_cast<int>(width() * 0.75));
 
   // adjust column width
-  dic_content_->setColumnWidth(0, static_cast<int>(width() * 0.20));
-  dic_content_->setColumnWidth(1, static_cast<int>(width() * 0.20));
-  dic_content_->setColumnWidth(2, static_cast<int>(width() * 0.15));
+  dic_content_->setColumnWidth(0, static_cast<int>(width() * 0.18));
+  dic_content_->setColumnWidth(1, static_cast<int>(width() * 0.18));
+  dic_content_->setColumnWidth(2, static_cast<int>(width() * 0.18));
 
   sort_state_.sorted = false;
 
@@ -1044,6 +1046,52 @@ void DictionaryTool::DeleteWord() {
   modified_ = true;
 }
 
+void DictionaryTool::EditPOS(const string &pos) {
+  const QList<QTableWidgetItem *> items =
+      dic_content_->selectedItems();
+  if (items.empty()) {
+    return;
+  }
+
+  const QString new_pos = QString::fromUtf8(pos.c_str());
+  setUpdatesEnabled(false);
+
+  for (int i = 0; i < items.size(); ++i) {
+    const int row = items[i]->row();
+    dic_content_->setItem(row, 2, new QTableWidgetItem(new_pos));
+  }
+
+  setUpdatesEnabled(true);
+  dic_content_->setEnabled(true);
+}
+
+void DictionaryTool::EditComment() {
+  const QList<QTableWidgetItem *> items =
+      dic_content_->selectedItems();
+  if (items.empty()) {
+    return;
+  }
+
+  bool ok = false;
+  const QString new_comment = QInputDialog::getText(
+      this, window_title_, tr("New comment"),
+      QLineEdit::Normal, "", &ok,
+      Qt::WindowTitleHint | Qt::WindowSystemMenuHint);
+
+  if (!ok) {
+    return;
+  }
+
+  setUpdatesEnabled(false);
+  for (int i = 0; i < items.size(); ++i) {
+    const int row = items[i]->row();
+    dic_content_->setItem(row, 3, new QTableWidgetItem(new_comment));
+  }
+
+  setUpdatesEnabled(true);
+  dic_content_->setEnabled(true);
+}
+
 void DictionaryTool::CloseWindow() {
   // move the focus to the close button to submit all incomplete inputs in
   // the word cells. http://b/211766
@@ -1110,12 +1158,30 @@ void DictionaryTool::OnContextMenuRequestedForContent(const QPoint &pos) {
   }
   QAction *delete_action = menu->addAction(delete_menu_text);
 
+  menu->addSeparator();
+  QMenu *sub_menu = menu->addMenu(tr("Change category to"));
+  vector<string> pos_list;
+  UserPOS::GetPOSList(&pos_list);
+  vector<QAction *> change_pos_actions(pos_list.size());
+  for (size_t i = 0; i < pos_list.size(); ++i) {
+    change_pos_actions[i] = sub_menu->addAction(
+        QString::fromUtf8(pos_list[i].c_str()));
+  }
+  QAction *edit_comment_action = menu->addAction(tr("Edit comment"));
   QAction *selected_action = menu->exec(QCursor::pos());
 
   if (selected_action == add_action) {
     AddWord();
   } else if (selected_action == delete_action) {
     DeleteWord();
+  } else if (selected_action == edit_comment_action) {
+    EditComment();
+  } else {
+    for (int i = 0; i < change_pos_actions.size(); ++i) {
+      if (selected_action == change_pos_actions[i]) {
+        EditPOS(pos_list[i]);
+      }
+    }
   }
 }
 
@@ -1260,6 +1326,11 @@ void DictionaryTool::ReportError() {
       QMessageBox::critical(this, window_title_,
                             tr("Dictionary name is too long."));
       break;
+    case UserDictionaryStorage::DUPLICATED_DICTIONARY_NAME:
+      LOG(ERROR) << "duplicated dictionary name";
+      QMessageBox::critical(this, window_title_,
+                            tr("Dictionary already exists."));
+      break;
     default:
       LOG(ERROR) << "A fatal error occurred";
       QMessageBox::critical(this, window_title_,
@@ -1300,13 +1371,11 @@ void DictionaryTool::SaveAndReloadServer() {
     return;
   }
 
-#ifndef OS_MACOSX
   // Update server version if need be.
   if (!session_->CheckVersionOrRestartServer()) {
     LOG(ERROR) << "CheckVersionOrRestartServer failed";
     return;
   }
-#endif
 
   // We don't show any dialog even when an error happens, since
   // dictionary serialization is finished correctly.
