@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -147,12 +147,10 @@ bool IsPeerValid(int socket, pid_t *pid) {
 #endif
 
 #ifdef OS_LINUX
-  // On ARM Linux, we do nothing and just return true since the platform (at
-  // least the qemu emulator) doesn't support the getsockopt(sock, SOL_SOCKET,
-  // SO_PEERCRED) system call.
+  // On ARM Linux, we do nothing and just return true since the platform
+  // sometimes doesn't support the getsockopt(sock, SOL_SOCKET, SO_PEERCRED)
+  // system call.
   // TODO(yusukes): Add implementation for ARM Linux.
-  // TODO(yusukes): Check if the real ChromeOS for ARM (on an actual hardware,
-  //                not on qemu-arm) supports the syscall.
 #ifndef __arm__
   struct ucred peer_cred;
   int peer_cred_len = sizeof(peer_cred);
@@ -217,21 +215,26 @@ bool RecvMessage(int socket,
     *last_ipc_error = IPC_UNKNOWN_ERROR;
     return false;
   }
-  if (IsReadTimeout(socket, timeout)) {
-    LOG(WARNING) << "Read timeout " << timeout;
-    *last_ipc_error = IPC_TIMEOUT_ERROR;
-    return false;
-  }
-  const ssize_t length = ::recv(socket, buf, *buf_length, 0);
-  *buf_length = (length > 0) ? length : 0;
-  if (length == 0) {
-    LOG(WARNING) << "Received 0 result. ignored";
-  }
-  if (length < 0) {
-    LOG(ERROR) << "an error occurred during recv(): " << strerror(errno);
-    *last_ipc_error = IPC_READ_ERROR;
-    return false;
-  }
+  ssize_t buf_left = *buf_length;
+  ssize_t read_length = 0;
+  *buf_length = 0;
+  do {
+    if (IsReadTimeout(socket, timeout)) {
+      LOG(WARNING) << "Read timeout " << timeout;
+      *last_ipc_error = IPC_TIMEOUT_ERROR;
+      return false;
+    }
+    read_length = ::recv(socket, buf, buf_left, 0);
+    if (read_length < 0) {
+      LOG(ERROR) << "an error occurred during recv(): " << strerror(errno);
+      *buf_length = 0;
+      *last_ipc_error = IPC_READ_ERROR;
+      return false;
+    }
+    *buf_length += read_length;
+    buf += read_length;
+    buf_left -= read_length;
+  } while (read_length != 0 && buf_left > 0);
   VLOG(1) << *buf_length << " bytes received";
   return true;
 }
@@ -249,6 +252,12 @@ void SetCloseOnExecFlag(int fd) {
                  << strerror(errno);
   }
 }
+
+// Returns true if address is in abstract namespace. See unix(7) on Linux for
+// details.
+bool IsAbstractSocket(const string& address) {
+  return (!address.empty()) && (address[0] == '\0');
+}
 }  // namespace
 
 // Client
@@ -311,6 +320,11 @@ void IPCClient::Init(const string &name, const string &server_path) {
                   reinterpret_cast<const sockaddr*>(&address),
                   sun_len) != 0 ||
         !IsPeerValid(socket_, &pid)) {
+      if ((errno == ENOTSOCK || errno == ECONNREFUSED) &&
+          !IsAbstractSocket(server_address)) {
+        // If abstract namepace is not enabled, recreate server_addresss path.
+        ::unlink(server_address.c_str());
+      }
       LOG(WARNING) << "connect failed: " << strerror(errno);
       connected_ = false;
       manager->Clear();
@@ -352,6 +366,15 @@ bool IPCClient::Call(const char *request_,
     LOG(ERROR) << "SendMessage failed";
     return false;
   }
+
+  // Half-close the socket so that mozc_server could know the length of the
+  // request data. Without this, RecvMessage() in mozc_server would fail with
+  // timeout.
+  // TODO(yusukes): It would be also possible to modify Send and RecvMessage()
+  // so that they send/recv the payload size explicitly together with the actual
+  // data. Will revisit later.
+  ::shutdown(socket_, SHUT_WR);
+
   if (!RecvMessage(socket_, response_, response_size, timeout,
                    &last_ipc_error_)) {
     LOG(ERROR) << "RecvMessage failed";
@@ -409,11 +432,15 @@ IPCServer::IPCServer(const string &name,
 #ifdef OS_MACOSX
   addr.sun_len = SUN_LEN(&addr);
   const size_t sun_len = sizeof(addr);
-  ::chmod(server_address_.c_str(), 0600);  // Linux does not use files for IPC.
 #else
   const size_t sun_len = sizeof(addr.sun_family) + server_address_.size();
 #endif
+  if (!IsAbstractSocket(server_address_)) {
+    // Linux does not use files for IPC.
+    ::chmod(server_address_.c_str(), 0600);
+  }
   if (::bind(socket_, reinterpret_cast<sockaddr *>(&addr), sun_len) != 0) {
+    // The UNIX domain socket file (server_address_) already exists?
     LOG(FATAL) << "bind() failed: " << strerror(errno);
     return;
   }
@@ -438,10 +465,10 @@ IPCServer::~IPCServer() {
   }
   ::shutdown(socket_, SHUT_RDWR);
   ::close(socket_);
-#ifdef OS_MACOSX
-  // Since Linux uses abstract namespace, unlink() is not necessary.
-  ::unlink(server_address_.c_str());
-#endif
+  if (!IsAbstractSocket(server_address_)) {
+    // When abstract namespace is used, unlink() is not necessary.
+    ::unlink(server_address_.c_str());
+  }
   connected_ = false;
   socket_ = kInvalidSocket;
   VLOG(1) << "IPCServer destructed";
@@ -487,10 +514,10 @@ void IPCServer::Loop() {
 
   ::shutdown(socket_, SHUT_RDWR);
   ::close(socket_);
-#ifdef OS_MACOSX
-  // Since Linux uses abstract namespace, unlink() is not necessary.
-  ::unlink(server_address_.c_str());
-#endif
+  if (!IsAbstractSocket(server_address_)) {
+    // When abstract namespace is used, unlink() is not necessary.
+    ::unlink(server_address_.c_str());
+  }
   connected_ = false;
   socket_ = kInvalidSocket;
 }
