@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -37,7 +37,8 @@
 #include <objc/objc-class.h>
 
 #include "base/mac_util.h"
-#include "client/session_interface.h"
+#include "base/util.h"
+#include "client/session_mock.h"
 #include "renderer/renderer_interface.h"
 #include "testing/base/public/googletest.h"
 #include "testing/base/public/gunit.h"
@@ -79,6 +80,7 @@
   NSRange expectedRange;
   string *selectedMode_;
   NSString *insertedText_;
+  NSString *overriddenLayout_;
   map<string, int> *counters_;
 }
 @property(readwrite, assign) NSString *bundleIdentifier;
@@ -86,6 +88,7 @@
 @property(readwrite, assign) NSRange expectedRange;
 @property(readonly) string *selectedMode;
 @property(readonly) NSString *insertedText;
+@property(readonly) NSString *overriddenLayout;
 @end
 
 @implementation MockClient
@@ -94,6 +97,7 @@
 @synthesize expectedRange;
 @synthesize selectedMode = selectedMode_;
 @synthesize insertedText = insertedText_;
+@synthesize overriddenLayout = overriddenLayout_;
 
 - (MockClient *)init {
   self = [super init];
@@ -109,6 +113,7 @@
   delete selectedMode_;
   [bundleIdentifier release];
   [insertedText_ release];
+  [overriddenLayout_ release];
   [super dealloc];
 }
 
@@ -149,6 +154,12 @@
   [insertedText_ release];
   insertedText_ = [result retain];
 }
+
+- (void)overrideKeyboardWithKeyboardNamed:(NSString *)newLayout {
+  (*counters_)["overrideKeyboardWithKeyboardNamed:"]++;
+  [overriddenLayout_ release];
+  overriddenLayout_ = [newLayout retain];
+}
 @end
 
 @interface MockScreen : NSScreen
@@ -168,6 +179,7 @@
 @end
 
 namespace {
+const NSTimeInterval kDoubleTapInterval = 0.5;
 int openURL_count = 0;
 BOOL openURL_test(id self, SEL selector, NSURL *url) {
   openURL_count++;
@@ -180,96 +192,6 @@ NSArray *dummy_screens(id self, SEL selector) {
 }  // anonymous namespace
 
 
-class MockSession : public mozc::client::SessionInterface {
- public:
-  MockSession() {}
-#define MockMethod(return_type, method_name, arg, body) \
-  virtual return_type method_name(arg) { \
-    call_counters_[#method_name]++; body; \
-  } \
-  int counter_##method_name() const { return call_counters_[#method_name]; }
-#define BoolMethod(method_name, arg) \
-  MockMethod(bool, method_name, arg, return true)
-#define VoidMethod(method_name, arg) \
-  MockMethod(void, method_name, arg, ;)
-
-  bool IsValidRunLevel() const { return true; }
-  BoolMethod(EnsureSession, void);
-  BoolMethod(EnsureConnection, void);
-  BoolMethod(CheckVersionOrRestartServer, void);
-  BoolMethod(ClearUserHistory, void);
-  BoolMethod(ClearUserPrediction, void);
-  BoolMethod(ClearUnusedUserPrediction, void);
-  BoolMethod(Shutdown, void);
-  BoolMethod(SyncData, void);
-  BoolMethod(Reload, void);
-  BoolMethod(NoOperation, void);
-  VoidMethod(set_timeout, int timeout);
-  VoidMethod(set_restricted, bool restricted);
-  VoidMethod(set_server_program, const string &server_program);
-  VoidMethod(Reset, void);
-  VoidMethod(EnableCascadingWindow, bool enable);
-  BoolMethod(Cleanup, void);
-
-  MockMethod(bool, GetConfig, mozc::config::Config *config, {
-      config->CopyFrom(expected_config_); return true;
-    });
-  MockMethod(bool, SetConfig, const mozc::config::Config &config, {
-      expected_config_.CopyFrom(config); return true;
-    });
-
-  MockMethod(void, set_client_capability,
-             const mozc::commands::Capability &capability, {
-               actual_capability_.CopyFrom(capability);
-             });
-  const mozc::commands::Capability &actual_capability() {
-    return actual_capability_;
-  }
-
-#undef VoidMethod
-#undef BoolMethod
-#undef MockMethod
-
-#define OutputMethod(method_name, arg_type) \
-  virtual bool method_name(arg_type arg, mozc::commands::Output *output) {   \
-      call_counters_[#method_name]++; \
-      called_##method_name##_.CopyFrom(arg); \
-      output->CopyFrom(outputs_[#method_name]); \
-      return true; \
-  } \
-  arg_type called_##method_name() const { return called_##method_name##_; } \
-  int counter_##method_name() const { return call_counters_[#method_name]; } \
-  void set_output_##method_name(const mozc::commands::Output &output) {   \
-      outputs_[#method_name].CopyFrom(output); \
-  }
-
-  OutputMethod(SendKey, const mozc::commands::KeyEvent &);
-  OutputMethod(TestSendKey, const mozc::commands::KeyEvent &);
-  OutputMethod(SendCommand, const mozc::commands::SessionCommand &);
-
-  bool LaunchTool(const string &mode, const string &extra_arg) {
-    return true;
-  }
-  bool OpenBrowser(const string &url) {
-    return true;
-  }
-
-  bool PingServer() const {
-    call_counters_["PingServer"]++;
-    return true;
-  }
-  int counter_PingServer() const { return call_counters_["PingServer"]; }
-
- private:
-  mutable map<string, int> call_counters_;
-  map<string, mozc::commands::Output> outputs_;
-  mozc::config::Config expected_config_;
-  mozc::commands::Capability actual_capability_;
-  mozc::commands::KeyEvent called_SendKey_;
-  mozc::commands::KeyEvent called_TestSendKey_;
-  mozc::commands::SessionCommand called_SendCommand_;
-};
-
 class MockRenderer : public mozc::renderer::RendererInterface {
  public:
   MockRenderer()
@@ -337,7 +259,7 @@ class GoogleJapaneseInputControllerTest : public testing::Test {
   void TearDown() {
     [controller_ release];
     [mock_server_ release];
-    // MockSession and MockRenderer are released during the release of
+    // SessionMock and MockRenderer are released during the release of
     // |controller_|.
     [pool_ release];
   }
@@ -348,7 +270,7 @@ class GoogleJapaneseInputControllerTest : public testing::Test {
                           delegate:nil
                             client:mock_client_];
     mock_server_.expectedController = controller_;
-    mock_session_ = new MockSession;
+    mock_session_ = new mozc::client::SessionMock;
     controller_.session = mock_session_;
     mock_renderer_ = new MockRenderer;
     controller_.renderer = mock_renderer_;
@@ -360,7 +282,7 @@ class GoogleJapaneseInputControllerTest : public testing::Test {
     SetUpController();
   }
 
-  MockSession *mock_session_;
+  mozc::client::SessionMock *mock_session_;
   MockClient *mock_client_;
   MockRenderer *mock_renderer_;
 
@@ -453,6 +375,27 @@ bool ComparePreedit(NSAttributedString *expected, NSAttributedString *actual) {
   return true;
 }
 
+NSTimeInterval GetDoubleTapInterval() {
+  // TODO(horo): This value should be get from system configuration.
+  return kDoubleTapInterval;
+}
+
+BOOL SendKanaKeyEvent(GoogleJapaneseInputController *controller,
+                      MockClient *client) {
+  // tap Kana-key
+  NSEvent *kanaKeyEvent = [NSEvent keyEventWithType:NSKeyDown
+                                           location:NSZeroPoint
+                                      modifierFlags:0
+                                          timestamp:0
+                                       windowNumber:0
+                                            context:nil
+                                         characters:@" "
+                        charactersIgnoringModifiers:@" "
+                                          isARepeat:NO
+                                            keyCode:kVK_JIS_Kana];
+  return [controller handleEvent:kanaKeyEvent client:client];
+}
+
 TEST_F(GoogleJapaneseInputControllerTest, UpdateComposedString) {
   // If preedit is NULL, it still calls setMarkedText, with an empty string.
   NSMutableAttributedString *expected =
@@ -508,7 +451,7 @@ TEST_F(GoogleJapaneseInputControllerTest, ClearCandidates) {
 
 TEST_F(GoogleJapaneseInputControllerTest, UpdateCandidates) {
   // When output is null, same as ClearCandidate
-  [controller_ updateCandidates:NULL client:mock_client_];
+  [controller_ updateCandidates:NULL];
   EXPECT_EQ(1, mock_renderer_->counter_ExecCommand());
   EXPECT_FALSE(mock_renderer_->CalledCommand().visible());
   EXPECT_EQ(0,
@@ -529,7 +472,7 @@ TEST_F(GoogleJapaneseInputControllerTest, UpdateCandidates) {
 
   // setup the cursor position
   mock_client_.expectedCursor = NSMakeRect(50, 50, 1, 10);
-  [controller_ updateCandidates:&output client:mock_client_];
+  [controller_ updateCandidates:&output];
   EXPECT_EQ(1,
             [mock_client_
               getCounter:"attributesForCharacterIndex:lineHeightRectangle:"]);
@@ -548,7 +491,7 @@ TEST_F(GoogleJapaneseInputControllerTest, UpdateCandidates) {
 
   // reshow the candidate window again -- but cursor position has changed.
   mock_client_.expectedCursor = NSMakeRect(60, 50, 1, 10);
-  [controller_ updateCandidates:&output client:mock_client_];
+  [controller_ updateCandidates:&output];
   // Does not change: not call again
   EXPECT_EQ(1,
             [mock_client_
@@ -562,7 +505,7 @@ TEST_F(GoogleJapaneseInputControllerTest, UpdateCandidates) {
 
   // Then update without candidate entries -- goes invisible.
   output.Clear();
-  [controller_ updateCandidates:&output client:mock_client_];
+  [controller_ updateCandidates:&output];
   // Does not change: not call again
   EXPECT_EQ(1,
             [mock_client_
@@ -585,24 +528,6 @@ TEST_F(GoogleJapaneseInputControllerTest, OpenLink) {
   EXPECT_EQ(1, openURL_count);
 }
 
-TEST_F(GoogleJapaneseInputControllerTest, HandleInputMode) {
-  // Does not support multiple-calculation
-  mozc::config::Config config;
-  config.set_preedit_method(mozc::config::Config::KANA);
-  config.set_yen_sign_character(mozc::config::Config::BACKSLASH);
-  mock_session_->SetConfig(config);
-
-  [controller_ handleInputMode];
-  EXPECT_EQ(1, mock_session_->counter_GetConfig());
-  EXPECT_EQ(KANA, controller_.keyCodeMap.inputMode);
-  EXPECT_EQ(mozc::config::Config::BACKSLASH, controller_.yenSignCharacter);
-
-  // If called twice, it does not change the state to reduce the call
-  // of GetConfig().
-  [controller_ handleInputMode];
-  EXPECT_EQ(1, mock_session_->counter_GetConfig());
-}
-
 TEST_F(GoogleJapaneseInputControllerTest, SwitchModeToDirect) {
   // setup the IME status
   controller_.mode = mozc::commands::HIRAGANA;
@@ -619,7 +544,7 @@ TEST_F(GoogleJapaneseInputControllerTest, SwitchModeToDirect) {
 
   [controller_ switchModeToDirect:mock_client_];
   EXPECT_EQ(mozc::commands::DIRECT, controller_.mode);
-  EXPECT_EQ(1, mock_session_->counter_SendKey());
+  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendKey"));
   EXPECT_TRUE(mock_session_->called_SendKey().has_special_key());
   EXPECT_EQ(mozc::commands::KeyEvent::OFF,
             mock_session_->called_SendKey().special_key());
@@ -634,11 +559,11 @@ TEST_F(GoogleJapaneseInputControllerTest, SwitchModeInternal) {
   controller_.mode = mozc::commands::DIRECT;
   [controller_ switchModeInternal:mozc::commands::HIRAGANA];
   EXPECT_EQ(mozc::commands::HIRAGANA, controller_.mode);
-  EXPECT_EQ(1, mock_session_->counter_SendKey());
+  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendKey"));
   EXPECT_TRUE(mock_session_->called_SendKey().has_special_key());
   EXPECT_EQ(mozc::commands::KeyEvent::ON,
             mock_session_->called_SendKey().special_key());
-  EXPECT_EQ(1, mock_session_->counter_SendCommand());
+  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendCommand"));
   EXPECT_EQ(mozc::commands::SessionCommand::SWITCH_INPUT_MODE,
             mock_session_->called_SendCommand().type());
   EXPECT_EQ(mozc::commands::HIRAGANA,
@@ -648,8 +573,8 @@ TEST_F(GoogleJapaneseInputControllerTest, SwitchModeInternal) {
   controller_.mode = mozc::commands::HIRAGANA;
   [controller_ switchModeInternal:mozc::commands::HALF_KATAKANA];
   EXPECT_EQ(mozc::commands::HALF_KATAKANA, controller_.mode);
-  EXPECT_EQ(1, mock_session_->counter_SendKey());
-  EXPECT_EQ(2, mock_session_->counter_SendCommand());
+  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(2, mock_session_->GetFunctionCallCount("SendCommand"));
   EXPECT_EQ(mozc::commands::SessionCommand::SWITCH_INPUT_MODE,
             mock_session_->called_SendCommand().type());
   EXPECT_EQ(mozc::commands::HALF_KATAKANA,
@@ -658,15 +583,242 @@ TEST_F(GoogleJapaneseInputControllerTest, SwitchModeInternal) {
   // going to same mode does not cause sendcommand
   [controller_ switchModeInternal:mozc::commands::HALF_KATAKANA];
   EXPECT_EQ(mozc::commands::HALF_KATAKANA, controller_.mode);
-  EXPECT_EQ(1, mock_session_->counter_SendKey());
-  EXPECT_EQ(2, mock_session_->counter_SendCommand());
+  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(2, mock_session_->GetFunctionCallCount("SendCommand"));
 }
 
 TEST_F(GoogleJapaneseInputControllerTest, SwitchDisplayMode) {
   EXPECT_TRUE(mock_client_.selectedMode->empty());
   EXPECT_EQ(mozc::commands::DIRECT, controller_.mode);
-  [controller_ switchDisplayMode:mock_client_];
+  [controller_ switchDisplayMode];
   EXPECT_EQ(1, [mock_client_ getCounter:"selectInputMode:"]);
   string expected = mozc::MacUtil::GetLabelForSuffix("Roman");
   EXPECT_EQ(expected, *(mock_client_.selectedMode));
+
+  // Does not change the display mode for MS Word.  See
+  // GoogleJapaneseInputController.mm for the detailed information.
+  ResetClientBundleIdentifier(@"com.microsoft.Word");
+  [controller_ switchMode:mozc::commands::HIRAGANA client:mock_client_];
+  EXPECT_EQ(mozc::commands::HIRAGANA, controller_.mode);
+  [controller_ switchDisplayMode];
+  // still remains 1 and display mode does not change.
+  EXPECT_EQ(1, [mock_client_ getCounter:"selectInputMode:"]);
+  expected = mozc::MacUtil::GetLabelForSuffix("Roman");
+  EXPECT_EQ(expected, *(mock_client_.selectedMode));
+}
+
+TEST_F(GoogleJapaneseInputControllerTest, commitText) {
+  controller_.replacementRange = NSMakeRange(0, 1);
+  [controller_ commitText:"foo" client:mock_client_];
+
+  EXPECT_EQ(1, [mock_client_ getCounter:"insertText:replacementRange:"]);
+  EXPECT_TRUE([@"foo" isEqualToString:mock_client_.insertedText]);
+  // location has to be cleared after the commit.
+  // Do not use NSNotFound directly in EXPECT_EQ because type checker
+  // gets confused for the comparision of enums.
+  int expected = NSNotFound;
+  EXPECT_EQ(expected, [controller_ replacementRange].location);
+}
+
+TEST_F(GoogleJapaneseInputControllerTest, handleConfig) {
+  // Does not support multiple-calculation
+  mozc::config::Config config;
+  config.set_preedit_method(mozc::config::Config::KANA);
+  config.set_yen_sign_character(mozc::config::Config::BACKSLASH);
+  config.set_use_japanese_layout(true);
+  mock_session_->SetConfig(config);
+  mock_session_->SetBoolFunctionReturn("GetConfig", true);
+
+  [controller_ handleConfig];
+  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("GetConfig"));
+  EXPECT_EQ(KANA, controller_.keyCodeMap.inputMode);
+  EXPECT_EQ(mozc::config::Config::BACKSLASH, controller_.yenSignCharacter);
+  EXPECT_EQ(1, [mock_client_ getCounter:"overrideKeyboardWithKeyboardNamed:"]);
+  EXPECT_TRUE([@"com.apple.keylayout.US"
+                isEqualToString:mock_client_.overriddenLayout])
+      << [mock_client_.overriddenLayout UTF8String];
+}
+
+TEST_F(GoogleJapaneseInputControllerTest, DoubleTapKanaUndo) {
+  // tap (short) tap -> emit undo command
+  controller_.mode = mozc::commands::HIRAGANA;
+
+  // set expectedRange for Undo
+  mock_client_.expectedRange = NSMakeRange(1, 0);
+
+  // Send Kana-key.
+  // Because of special hack for Eisu/Kana keys, it returns YES.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it did not send command.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+
+  // Sleep less than DoubleTapInterval (sec)
+  mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
+
+  // Send Kana-key.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it sent an undo command.
+  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
+            mock_session_->called_SendCommand().type());
+}
+
+TEST_F(GoogleJapaneseInputControllerTest, DoubleTapKanaUndoTimeOver) {
+ // tap (long) tap -> don't emit undo command
+  controller_.mode = mozc::commands::HIRAGANA;
+
+  // set expectedRange for Undo
+  mock_client_.expectedRange = NSMakeRange(1, 0);
+
+  // Send Kana-key.
+  // Because of special hack for Eisu/Kana keys, it returns YES.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it did not send command.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+
+  // Sleep more than DoubleTapInterval (sec)
+  mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 * 2.0);
+
+  // Send Kana-key.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it did not send command.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+}
+
+TEST_F(GoogleJapaneseInputControllerTest, SingleAndDoubleTapKanaUndo) {
+  // tap (long) tap (short) tap -> emit once
+  controller_.mode = mozc::commands::HIRAGANA;
+
+  // set expectedRange for Undo
+  mock_client_.expectedRange = NSMakeRange(1, 0);
+
+  // Send Kana-key.
+  // Because of special hack for Eisu/Kana keys, it returns YES.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it did not send command.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+
+  // Sleep more than DoubleTapInterval (sec)
+  mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 * 2.0);
+
+  // Send Kana-key.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it did not send command.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+
+  // Sleep less than DoubleTapInterval (sec)
+  mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
+
+  // Send Kana-key.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it sent an undo command.
+  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
+            mock_session_->called_SendCommand().type());
+}
+
+TEST_F(GoogleJapaneseInputControllerTest, TripleTapKanaUndo) {
+  // tap (short) tap (short) tap -> emit twice
+  controller_.mode = mozc::commands::HIRAGANA;
+
+  // set expectedRange for Undo
+  mock_client_.expectedRange = NSMakeRange(1, 0);
+
+  // Send Kana-key.
+  // Because of special hack for Eisu/Kana keys, it returns YES.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it did not send command.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+
+  // Sleep less than DoubleTapInterval (sec)
+  mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
+
+  // Send Kana-key.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it sent an undo command.
+  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
+            mock_session_->called_SendCommand().type());
+
+  // Sleep less than DoubleTapInterval (sec)
+  mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
+
+  // Send Kana-key.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it sent an undo command.
+  EXPECT_EQ(2, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
+            mock_session_->called_SendCommand().type());
+}
+
+TEST_F(GoogleJapaneseInputControllerTest, QuadrupleTapKanaUndo) {
+  // tap (short) tap (short) tap (short) tap -> emit thrice
+  controller_.mode = mozc::commands::HIRAGANA;
+
+  // set expectedRange for Undo
+  mock_client_.expectedRange = NSMakeRange(1, 0);
+
+  // Send Kana-key.
+  // Because of special hack for Eisu/Kana keys, it returns YES.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it did not send command.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+
+  // Sleep less than DoubleTapInterval (sec)
+  mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
+
+  // Send Kana-key.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it sent an undo command.
+  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
+            mock_session_->called_SendCommand().type());
+
+  // Sleep less than DoubleTapInterval (sec)
+  mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
+
+  // Send Kana-key.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it sent an undo command.
+  EXPECT_EQ(2, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
+            mock_session_->called_SendCommand().type());
+
+  // Sleep less than DoubleTapInterval (sec)
+  mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
+
+  // Send Kana-key.
+  EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
+  // Check if it did not send key.
+  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  // Check if it sent an undo command.
+  EXPECT_EQ(3, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
+            mock_session_->called_SendCommand().type());  
 }
