@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -32,22 +32,25 @@
 #include "base/base.h"
 #include "base/singleton.h"
 #include "converter/segments.h"
+#include "session/commands.pb.h"
 #include "session/config_handler.h"
 #include "session/config.pb.h"
-#include "prediction/predictor_interface.h"
 #include "prediction/dictionary_predictor.h"
+#include "prediction/predictor_interface.h"
 #include "prediction/user_history_predictor.h"
 
 namespace mozc {
 namespace {
 const int kPredictionSize = 100;
 
-class PredictorImpl : public PredictorInterface {
- public:
-  PredictorImpl();
-  virtual ~PredictorImpl();
 
-  virtual bool Predict(Segments *segments) const;
+class BasePredictor : public PredictorInterface {
+ public:
+  BasePredictor();
+  virtual ~BasePredictor();
+
+  // Overwrite predictor
+  virtual bool Predict(Segments *segments) const = 0;
 
   // Hook(s) for all mutable operations
   virtual void Finish(Segments *segments);
@@ -64,12 +67,16 @@ class PredictorImpl : public PredictorInterface {
   // Sync user history
   virtual bool Sync();
 
- public:
-  UserHistoryPredictor *user_history_predictor_;
-  DictionaryPredictor  *dictionary_predictor_;
-  vector<PredictorInterface *> predictors_;
+  // Reload usre history
+  virtual bool Reload();
 };
 
+class DefaultPredictor : public BasePredictor {
+ public:
+  virtual bool Predict(Segments *segments) const;
+};
+
+
 size_t GetCandidatesSize(const Segments &segments) {
   if (segments.conversion_segments_size() <= 0) {
     LOG(ERROR) << "No conversion segments found";
@@ -78,25 +85,14 @@ size_t GetCandidatesSize(const Segments &segments) {
   return segments.conversion_segment(0).candidates_size();
 }
 
-PredictorImpl::PredictorImpl()
-    : user_history_predictor_(new UserHistoryPredictor),
-      dictionary_predictor_(new DictionaryPredictor) {
-  predictors_.push_back(user_history_predictor_);
-  predictors_.push_back(dictionary_predictor_);
-  DCHECK(user_history_predictor_);
-  DCHECK(dictionary_predictor_);
-}
+BasePredictor::BasePredictor() {}
 
-PredictorImpl::~PredictorImpl() {
-  delete user_history_predictor_;
-  delete dictionary_predictor_;
-  predictors_.clear();
-}
+BasePredictor::~BasePredictor() {}
+
+void BasePredictor::Finish(Segments *segments) {
+  PredictorFactory::GetUserHistoryPredictor()->Finish(segments);
+  PredictorFactory::GetDictionaryPredictor()->Finish(segments);
 
-void PredictorImpl::Finish(Segments *segments) {
-  for (size_t i = 0; i < predictors_.size(); ++i) {
-    predictors_[i]->Finish(segments);
-  }
   if (segments->conversion_segments_size() < 1 ||
       segments->request_type() == Segments::CONVERSION) {
     return;
@@ -113,56 +109,116 @@ void PredictorImpl::Finish(Segments *segments) {
   segment->set_key(segment->candidate(0).key);
 }
 
-void PredictorImpl::Revert(Segments *segments) {
-  for (size_t i = 0; i < predictors_.size(); ++i) {
-    predictors_[i]->Revert(segments);
-  }
+// Since DictionaryPredictor is immutable, no need
+// to call DictionaryPredictor::Revert/Clear*/Finish methods.
+void BasePredictor::Revert(Segments *segments) {
+  PredictorInterface *user_history_predictor =
+      PredictorFactory::GetUserHistoryPredictor();
+  DCHECK(user_history_predictor != NULL);
+  user_history_predictor->Revert(segments);
 }
 
-bool PredictorImpl::Predict(Segments *segments) const {
-  bool result = false;
+bool BasePredictor::ClearAllHistory() {
+  PredictorInterface *user_history_predictor =
+      PredictorFactory::GetUserHistoryPredictor();
+  DCHECK(user_history_predictor != NULL);
+  return user_history_predictor->ClearAllHistory();
+}
 
+bool BasePredictor::ClearUnusedHistory() {
+  PredictorInterface *user_history_predictor =
+      PredictorFactory::GetUserHistoryPredictor();
+  DCHECK(user_history_predictor);
+  return user_history_predictor->ClearUnusedHistory();
+}
+
+bool BasePredictor::Sync() {
+  PredictorInterface *user_history_predictor =
+      PredictorFactory::GetUserHistoryPredictor();
+  DCHECK(user_history_predictor);
+  return user_history_predictor->Sync();
+}
+
+bool BasePredictor::Reload() {
+  PredictorInterface *user_history_predictor =
+      PredictorFactory::GetUserHistoryPredictor();
+  DCHECK(user_history_predictor);
+  return user_history_predictor->Reload();
+}
+
+bool DefaultPredictor::Predict(Segments *segments) const {
   int size = kPredictionSize;
   if (segments->request_type() == Segments::SUGGESTION) {
     size = min(9, max(1, static_cast<int>(GET_CONFIG(suggestions_size))));
   }
 
-  for (size_t i = 0; i < predictors_.size(); ++i) {
-    if (size <= 0) {
-      break;
-    }
-    segments->set_max_prediction_candidates_size(static_cast<size_t>(size));
-    result |= predictors_[i]->Predict(segments);
-    size -= static_cast<int>(GetCandidatesSize(*segments));
+  bool result = false;
+
+  PredictorInterface *user_history_predictor =
+      PredictorFactory::GetUserHistoryPredictor();
+  PredictorInterface *dictionary_predictor =
+      PredictorFactory::GetDictionaryPredictor();
+  DCHECK(user_history_predictor);
+  DCHECK(dictionary_predictor);
+
+  int remained_size = size;
+  segments->set_max_prediction_candidates_size(static_cast<size_t>(size));
+  result |= user_history_predictor->Predict(segments);
+  remained_size = size - static_cast<size_t>(GetCandidatesSize(*segments));
+
+  // Do not call dictionary_predictor if the size of candidates get
+  // >= suggestions_size.
+  if (remained_size <= 0) {
+    return result;
   }
 
+  segments->set_max_prediction_candidates_size(remained_size);
+  result |= dictionary_predictor->Predict(segments);
+  remained_size = size - static_cast<size_t>(GetCandidatesSize(*segments));
+
+
   return result;
 }
 
-bool PredictorImpl::ClearAllHistory() {
-  return user_history_predictor_->ClearAllHistory();
-}
-
-bool PredictorImpl::ClearUnusedHistory() {
-  return user_history_predictor_->ClearUnusedHistory();
-}
-
-bool PredictorImpl::Sync() {
-  return user_history_predictor_->Sync();
-}
 
 PredictorInterface *g_predictor = NULL;
+PredictorInterface *g_user_history_predictor = NULL;
+PredictorInterface *g_dictionary_predictor = NULL;
+
 }  // namespace
 
+#define GET_PREDICTOR(PredictorClass, predictor_instance) do { \
+  if (predictor_instance == NULL) { \
+    return Singleton<PredictorClass>::get(); \
+  } else { \
+    return predictor_instance; \
+  } \
+} while (0)
+
 PredictorInterface *PredictorFactory::GetPredictor() {
-  if (g_predictor == NULL) {
-    return Singleton<PredictorImpl>::get();
-  } else {
-    return g_predictor;
-  }
+    GET_PREDICTOR(DefaultPredictor, g_predictor);
 }
 
 void PredictorFactory::SetPredictor(PredictorInterface *predictor) {
   g_predictor = predictor;
 }
+
+PredictorInterface *PredictorFactory::GetUserHistoryPredictor() {
+  GET_PREDICTOR(UserHistoryPredictor, g_user_history_predictor);
+}
+
+void PredictorFactory::SetUserHistoryPredictor(PredictorInterface *predictor) {
+  g_user_history_predictor = predictor;
+}
+
+PredictorInterface *PredictorFactory::GetDictionaryPredictor() {
+  GET_PREDICTOR(DictionaryPredictor, g_dictionary_predictor);
+}
+
+void PredictorFactory::SetDictionaryPredictor(PredictorInterface *predictor) {
+  g_dictionary_predictor = predictor;
+}
+
+
+#undef GET_PREDICTOR
 }  // namespace mozc
