@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -34,16 +34,41 @@
 #include "converter/converter_interface.h"
 #include "converter/converter_mock.h"
 #include "converter/segments.h"
+#include "session/commands.pb.h"
 #include "session/config.pb.h"
 #include "session/config_handler.h"
 #include "testing/base/public/googletest.h"
 #include "testing/base/public/gunit.h"
 
+
 DECLARE_string(test_tmpdir);
 
 namespace mozc {
 namespace {
 
+class UserHistoryPredictorTest : public testing::Test {
+ protected:
+  virtual void SetUp() {
+    kUseMockPasswordManager = true;
+    Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+    config::ConfigHandler::GetDefaultConfig(&default_config_);
+    config::ConfigHandler::SetConfig(default_config_);
+  }
+
+  virtual void TearDown() {
+    config::ConfigHandler::SetConfig(default_config_);
+  }
+
+  virtual void EnableZeroQuerySuggestion() {
+  }
+
+  virtual void DisableZeroQuerySuggestion() {
+  }
+
+ private:
+  config::Config default_config_;
+};
+
 void MakeSegmentsForSuggestion(const string key,
                                Segments *segments) {
   segments->set_max_prediction_candidates_size(10);
@@ -56,7 +81,7 @@ void MakeSegmentsForSuggestion(const string key,
 void MakeSegmentsForPrediction(const string key,
                                Segments *segments) {
   segments->set_max_prediction_candidates_size(10);
-  segments->set_request_type(Segments::SUGGESTION);
+  segments->set_request_type(Segments::PREDICTION);
   Segment *seg = segments->add_segment();
   seg->set_key(key);
   seg->set_segment_type(Segment::FIXED_VALUE);
@@ -106,10 +131,20 @@ void AddCandidateWithDescription(const string &value,
   AddCandidateWithDescription(0, value, desc, segments);
 }
 
-TEST(UserHistoryPredictor, UserHistoryPredictorTest) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+void PrependHistorySegments(const string &key,
+                            const string &value,
+                            Segments *segments) {
+  Segment *seg = segments->push_front_segment();
+  seg->set_segment_type(Segment::HISTORY);
+  seg->set_key(key);
+  Segment::Candidate *c = seg->add_candidate();
+  c->key = key;
+  c->content_key = key;
+  c->value = value;
+  c->content_value = value;
+}
 
+TEST_F(UserHistoryPredictorTest, UserHistoryPredictorTest) {
   {
     UserHistoryPredictor predictor;
     predictor.WaitForSyncer();
@@ -192,7 +227,7 @@ TEST(UserHistoryPredictor, UserHistoryPredictorTest) {
 
       // "わたしの"
       MakeSegmentsForSuggestion(
-        "\xE3\x82\x8F\xE3\x81\x9F\xE3\x81\x97\xE3\x81\xAE", &segments);
+          "\xE3\x82\x8F\xE3\x81\x9F\xE3\x81\x97\xE3\x81\xAE", &segments);
       EXPECT_FALSE(predictor.Predict(&segments));
 
       config.set_use_history_suggest(true);
@@ -201,7 +236,7 @@ TEST(UserHistoryPredictor, UserHistoryPredictorTest) {
 
       // "わたしの"
       MakeSegmentsForSuggestion(
-        "\xE3\x82\x8F\xE3\x81\x9F\xE3\x81\x97\xE3\x81\xAE", &segments);
+          "\xE3\x82\x8F\xE3\x81\x9F\xE3\x81\x97\xE3\x81\xAE", &segments);
       EXPECT_FALSE(predictor.Predict(&segments));
     }
 
@@ -231,7 +266,7 @@ TEST(UserHistoryPredictor, UserHistoryPredictorTest) {
               "\xE7\xA7\x81\xE3\x81\xAE\xE5\x90\x8D\xE5\x89\x8D"
               "\xE3\x81\xAF\xE4\xB8\xAD\xE9\x87\x8E\xE3\x81\xA7\xE3\x81\x99");
 
-    //Exact Match
+    // Exact Match
     segments.Clear();
     // "わたしのなまえはなかのです"
     MakeSegmentsForSuggestion(
@@ -259,6 +294,7 @@ TEST(UserHistoryPredictor, UserHistoryPredictorTest) {
 
     // clear
     predictor.ClearAllHistory();
+    predictor.WaitForSyncer();
   }
 
   // nothing happen
@@ -298,10 +334,7 @@ TEST(UserHistoryPredictor, UserHistoryPredictorTest) {
   }
 }
 
-TEST(UserHistoryPredictor, DescriptionTest) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
-
+TEST_F(UserHistoryPredictorTest, DescriptionTest) {
   {
     UserHistoryPredictor predictor;
     predictor.WaitForSyncer();
@@ -374,7 +407,7 @@ TEST(UserHistoryPredictor, DescriptionTest) {
 
       // "わたしの"
       MakeSegmentsForSuggestion(
-        "\xE3\x82\x8F\xE3\x81\x9F\xE3\x81\x97\xE3\x81\xAE", &segments);
+          "\xE3\x82\x8F\xE3\x81\x9F\xE3\x81\x97\xE3\x81\xAE", &segments);
       EXPECT_FALSE(predictor.Predict(&segments));
 
       config.set_use_history_suggest(true);
@@ -383,7 +416,7 @@ TEST(UserHistoryPredictor, DescriptionTest) {
 
       // "わたしの"
       MakeSegmentsForSuggestion(
-        "\xE3\x82\x8F\xE3\x81\x9F\xE3\x81\x97\xE3\x81\xAE", &segments);
+          "\xE3\x82\x8F\xE3\x81\x9F\xE3\x81\x97\xE3\x81\xAE", &segments);
       EXPECT_FALSE(predictor.Predict(&segments));
     }
 
@@ -420,7 +453,7 @@ TEST(UserHistoryPredictor, DescriptionTest) {
               // "テスト"
               "\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88");
 
-    //Exact Match
+    // Exact Match
     segments.Clear();
     // "わたしのなまえはなかのです"
     MakeSegmentsForSuggestion(
@@ -454,6 +487,7 @@ TEST(UserHistoryPredictor, DescriptionTest) {
 
     // clear
     predictor.ClearAllHistory();
+    predictor.WaitForSyncer();
   }
 
   // nothing happen
@@ -493,10 +527,7 @@ TEST(UserHistoryPredictor, DescriptionTest) {
   }
 }
 
-TEST(UserHistoryPredictor, UserHistoryPredictorUnusedHistoryTest) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
-
+TEST_F(UserHistoryPredictorTest, UserHistoryPredictorUnusedHistoryTest) {
   {
     UserHistoryPredictor predictor;
     predictor.WaitForSyncer();
@@ -602,12 +633,11 @@ TEST(UserHistoryPredictor, UserHistoryPredictorUnusedHistoryTest) {
   }
 }
 
-TEST(UserHistoryPredictor, UserHistoryPredictorRevertTest) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+TEST_F(UserHistoryPredictorTest, UserHistoryPredictorRevertTest) {
   UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
   predictor.ClearAllHistory();
+  predictor.WaitForSyncer();
 
   Segments segments, segments2;
   // "わたしのなまえはなかのです"
@@ -638,7 +668,7 @@ TEST(UserHistoryPredictor, UserHistoryPredictorRevertTest) {
   predictor.Revert(&segments);
 
   segments.Clear();
-    // "わたしの"
+  // "わたしの"
   MakeSegmentsForSuggestion("\xE3\x82\x8F\xE3\x81\x9F"
                             "\xE3\x81\x97\xE3\x81\xAE", &segments);
 
@@ -649,50 +679,50 @@ TEST(UserHistoryPredictor, UserHistoryPredictorRevertTest) {
   EXPECT_EQ(segments.segment(0).candidates_size(), 0);
 }
 
-TEST(UserHistoryPredictor, UserHistoryPredictorClearTest) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
-
+TEST_F(UserHistoryPredictorTest, UserHistoryPredictorClearTest) {
   UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
 
-  // input "012-3456" 10 times
+  // input "testtest" 10 times
   for (int i = 0; i < 10; ++i) {
     Segments segments;
-    MakeSegmentsForConversion("012-3456", &segments);
-    AddCandidate("012-3456", &segments);
+    MakeSegmentsForConversion("testtest", &segments);
+    // "テストテスト"
+    AddCandidate("\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88"
+                 "\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88", &segments);
     predictor.Finish(&segments);
   }
 
   predictor.ClearAllHistory();
   predictor.WaitForSyncer();
 
-  // input "012-3456" 1 time
+  // input "testtest" 1 time
   for (int i = 0; i < 1; ++i) {
     Segments segments;
-    MakeSegmentsForConversion("012-3456", &segments);
-    AddCandidate("012-3456", &segments);
+    MakeSegmentsForConversion("testtest", &segments);
+    // "テストテスト"
+    AddCandidate("\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88"
+                 "\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88", &segments);
     predictor.Finish(&segments);
   }
 
   // frequency is cleared as well.
   {
     Segments segments;
-    MakeSegmentsForSuggestion("0", &segments);
+    MakeSegmentsForSuggestion("t", &segments);
     EXPECT_FALSE(predictor.Predict(&segments));
 
-    segments.clear();
-    MakeSegmentsForSuggestion("012-345", &segments);
+    segments.Clear();
+    MakeSegmentsForSuggestion("testte", &segments);
     EXPECT_TRUE(predictor.Predict(&segments));
   }
 }
 
-TEST(UserHistoryPredictor, UserHistoryPredictorTailingPunctuation) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+TEST_F(UserHistoryPredictorTest, UserHistoryPredictorTailingPunctuation) {
   UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
   predictor.ClearAllHistory();
+  predictor.WaitForSyncer();
 
   Segments segments;
 
@@ -750,12 +780,11 @@ TEST(UserHistoryPredictor, UserHistoryPredictorTailingPunctuation) {
             "\xE3\x80\x82");
 }
 
-TEST(UserHistoryPredictor, UserHistoryPredictorPreceedingPunctuation) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+TEST_F(UserHistoryPredictorTest, UserHistoryPredictorPreceedingPunctuation) {
   UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
   predictor.ClearAllHistory();
+  predictor.WaitForSyncer();
 
   Segments segments;
 
@@ -804,12 +833,12 @@ TEST(UserHistoryPredictor, UserHistoryPredictorPreceedingPunctuation) {
             "\xE3\x81\xAF\xE4\xB8\xAD\xE9\x87\x8E\xE3\x81\xA7\xE3\x81\x99");
 }
 
-TEST(UserHistoryPredictor, MultiSegmentsMultiInput) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+
+TEST_F(UserHistoryPredictorTest, MultiSegmentsMultiInput) {
   UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
   predictor.ClearAllHistory();
+  predictor.WaitForSyncer();
 
   Segments segments;
 
@@ -929,12 +958,11 @@ TEST(UserHistoryPredictor, MultiSegmentsMultiInput) {
             "\xE8\x89\xAF\xE5\xAD\x90\xE3\x81\xAB");
 }
 
-TEST(UserHistoryPredictor, MultiSegmentsSingleInput) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+TEST_F(UserHistoryPredictorTest, MultiSegmentsSingleInput) {
   UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
   predictor.ClearAllHistory();
+  predictor.WaitForSyncer();
 
   Segments segments;
 
@@ -1044,12 +1072,11 @@ TEST(UserHistoryPredictor, MultiSegmentsSingleInput) {
             "\xE8\x89\xAF\xE5\xAD\x90\xE3\x81\xAB");
 }
 
-TEST(UserHistoryPredictor, Regression2843371_Case1) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+TEST_F(UserHistoryPredictorTest, Regression2843371_Case1) {
   UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
   predictor.ClearAllHistory();
+  predictor.WaitForSyncer();
 
   Segments segments;
 
@@ -1124,15 +1151,13 @@ TEST(UserHistoryPredictor, Regression2843371_Case1) {
             "\xE3\x81\xAF\xE3\x80\x81"
             "\xE6\x97\xA5\xE6\x9C\xAC"
             "\xE3\x81\xA7\xE3\x81\x99");
-
 }
 
-TEST(UserHistoryPredictor, Regression2843371_Case2) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+TEST_F(UserHistoryPredictorTest, Regression2843371_Case2) {
   UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
   predictor.ClearAllHistory();
+  predictor.WaitForSyncer();
 
   Segments segments;
 
@@ -1145,7 +1170,9 @@ TEST(UserHistoryPredictor, Regression2843371_Case2) {
   AddCandidate(1, "(", &segments);
 
   // "とうきょう/東京"
-  MakeSegmentsForConversion("\xE3\x81\xA8\xE3\x81\x86\xE3\x81\x8D\xE3\x82\x87\xE3\x81\x86", &segments);
+  MakeSegmentsForConversion(
+      "\xE3\x81\xA8\xE3\x81\x86\xE3\x81\x8D\xE3\x82\x87\xE3\x81\x86",
+      &segments);
   AddCandidate(2, "\xE6\x9D\xB1\xE4\xBA\xAC", &segments);
 
   // ")"
@@ -1202,12 +1229,11 @@ TEST(UserHistoryPredictor, Regression2843371_Case2) {
             "\xE6\xB1\x9F\xE6\x88\xB8(\xE6\x9D\xB1\xE4\xBA\xAC");
 }
 
-TEST(UserHistoryPredictor, Regression2843371_Case3) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+TEST_F(UserHistoryPredictorTest, Regression2843371_Case3) {
   UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
   predictor.ClearAllHistory();
+  predictor.WaitForSyncer();
 
   Segments segments;
 
@@ -1282,12 +1308,11 @@ TEST(UserHistoryPredictor, Regression2843371_Case3) {
             "\xE3\x81\xAF\xE9\xAB\x98\xE3\x81\x84");
 }
 
-TEST(UserHistoryPredictor, Regression2843775) {
-  kUseMockPasswordManager = true;
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
+TEST_F(UserHistoryPredictorTest, Regression2843775) {
   UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
   predictor.ClearAllHistory();
+  predictor.WaitForSyncer();
 
   Segments segments;
 
@@ -1333,6 +1358,90 @@ TEST(UserHistoryPredictor, Regression2843775) {
             "\xE3\x81\x99");
 }
 
+TEST_F(UserHistoryPredictorTest, DuplicateString) {
+  UserHistoryPredictor predictor;
+  predictor.WaitForSyncer();
+  predictor.ClearAllHistory();
+  predictor.WaitForSyncer();
+
+  Segments segments;
+
+  // "らいおん/ライオン"
+  MakeSegmentsForConversion("\xE3\x82\x89\xE3\x81\x84\xE3\x81\x8A\xE3\x82\x93",
+                            &segments);
+  AddCandidate(0, "\xE3\x83\xA9\xE3\x82\xA4\xE3\x82\xAA\xE3\x83\xB3",
+               &segments);
+
+  // "（/（"
+  MakeSegmentsForConversion("\xEF\xBC\x88", &segments);
+  AddCandidate(1, "\xEF\xBC\x88", &segments);
+
+  // "もうじゅう/猛獣"
+  MakeSegmentsForConversion(
+      "\xE3\x82\x82\xE3\x81\x86\xE3\x81\x98\xE3\x82\x85\xE3\x81\x86",
+      &segments);
+  AddCandidate(2, "\xE7\x8C\x9B\xE7\x8D\xA3", &segments);
+
+  // "）と/）と"
+  MakeSegmentsForConversion("\xEF\xBC\x89\xE3\x81\xA8", &segments);
+  AddCandidate(3, "\xEF\xBC\x89\xE3\x81\xA8", &segments);
+
+  // "ぞうりむし/ゾウリムシ"
+  MakeSegmentsForConversion(
+      "\xE3\x81\x9E\xE3\x81\x86\xE3\x82\x8A\xE3\x82\x80\xE3\x81\x97",
+      &segments);
+  AddCandidate(
+      4,
+      "\xE3\x82\xBE\xE3\x82\xA6\xE3\x83\xAA\xE3\x83\xA0\xE3\x82\xB7",
+      &segments);
+
+  // "（/（"
+  MakeSegmentsForConversion("\xEF\xBC\x88", &segments);
+  AddCandidate(5, "\xEF\xBC\x88", &segments);
+
+  // "びせいぶつ/微生物"
+  MakeSegmentsForConversion(
+      "\xE3\x81\xB3\xE3\x81\x9B\xE3\x81\x84\xE3\x81\xB6\xE3\x81\xA4",
+      &segments);
+  AddCandidate(6, "\xE5\xBE\xAE\xE7\x94\x9F\xE7\x89\xA9", &segments);
+
+  // "）/）"
+  MakeSegmentsForConversion("\xEF\xBC\x89", &segments);
+  AddCandidate(7, "\xEF\xBC\x89", &segments);
+
+  predictor.Finish(&segments);
+
+  segments.Clear();
+
+  // "ぞうりむし"
+  MakeSegmentsForSuggestion(
+      "\xE3\x81\x9E\xE3\x81\x86\xE3\x82\x8A\xE3\x82\x80\xE3\x81\x97",
+      &segments);
+  EXPECT_TRUE(predictor.Predict(&segments));
+
+  for (int i = 0; i < segments.segment(0).candidates_size(); ++i) {
+    EXPECT_EQ(string::npos,
+              segments.segment(0).candidate(i).value.find(
+                  "\xE7\x8C\x9B\xE7\x8D\xA3"));  // "猛獣" should not be found
+  }
+
+  segments.Clear();
+
+  // "らいおん"
+  MakeSegmentsForSuggestion(
+      "\xE3\x82\x89\xE3\x81\x84\xE3\x81\x8A\xE3\x82\x93",
+      &segments);
+  EXPECT_TRUE(predictor.Predict(&segments));
+
+  // "ライオン（微生物" should not be found
+  for (int i = 0; i < segments.segment(0).candidates_size(); ++i) {
+    EXPECT_EQ(string::npos,
+              segments.segment(0).candidate(i).value.find(
+                  "\xE3\x83\xA9\xE3\x82\xA4\xE3\x82\xAA\xE3\x83\xB3"
+                  "\xEF\xBC\x88\xE5\xBE\xAE\xE7\x94\x9F\xE7\x89\xA9"));
+  }
+}
+
 struct Command {
   enum Type {
     LOOKUP,
@@ -1350,9 +1459,7 @@ int Random(int size) {
   return static_cast<int> (1.0 * size * rand() / (RAND_MAX + 1.0));
 }
 
-TEST(UserHistoryPredictor, SyncTest) {
-  Util::SetUserProfileDirectory(FLAGS_test_tmpdir);
-
+TEST_F(UserHistoryPredictorTest, SyncTest) {
   UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
 
@@ -1383,13 +1490,13 @@ TEST(UserHistoryPredictor, SyncTest) {
         predictor.WaitForSyncer();
         break;
       case Command::INSERT:
-        segments.clear();
+        segments.Clear();
         MakeSegmentsForConversion(commands[i].key, &segments);
         AddCandidate(commands[i].value, &segments);
         predictor.Finish(&segments);
         break;
       case Command::LOOKUP:
-        segments.clear();
+        segments.Clear();
         MakeSegmentsForSuggestion(commands[i].key, &segments);
         predictor.Predict(&segments);
       default:
@@ -1398,16 +1505,16 @@ TEST(UserHistoryPredictor, SyncTest) {
   }
 }
 
-TEST(UserHistoryPredictor, GetMatchTypeTest) {
-  EXPECT_EQ(UserHistoryPredictor::NO_MATCH,
-            UserHistoryPredictor::GetMatchType("", "test"));
-
+TEST_F(UserHistoryPredictorTest, GetMatchTypeTest) {
   EXPECT_EQ(UserHistoryPredictor::NO_MATCH,
             UserHistoryPredictor::GetMatchType("test", ""));
 
   EXPECT_EQ(UserHistoryPredictor::NO_MATCH,
             UserHistoryPredictor::GetMatchType("", ""));
 
+  EXPECT_EQ(UserHistoryPredictor::LEFT_EMPTY_MATCH,
+            UserHistoryPredictor::GetMatchType("", "test"));
+
   EXPECT_EQ(UserHistoryPredictor::NO_MATCH,
             UserHistoryPredictor::GetMatchType("foo", "bar"));
 
@@ -1421,7 +1528,7 @@ TEST(UserHistoryPredictor, GetMatchTypeTest) {
             UserHistoryPredictor::GetMatchType("foobar", "foo"));
 }
 
-TEST(UserHistoryPredictor, FingerPrintTest) {
+TEST_F(UserHistoryPredictorTest, FingerPrintTest) {
   const char kKey[] = "abc";
   const char kValue[] = "ABC";
 
@@ -1434,6 +1541,21 @@ TEST(UserHistoryPredictor, FingerPrintTest) {
   const uint32 entry_fp2 =
       UserHistoryPredictor::EntryFingerprint(entry);
 
+  const uint32 entry_fp3 =
+      UserHistoryPredictor::Fingerprint(
+          kKey, kValue,
+          UserHistoryPredictor::Entry::DEFAULT_ENTRY);
+
+  const uint32 entry_fp4 =
+      UserHistoryPredictor::Fingerprint(
+          kKey, kValue,
+          UserHistoryPredictor::Entry::CLEAN_ALL_EVENT);
+
+  const uint32 entry_fp5 =
+      UserHistoryPredictor::Fingerprint(
+          kKey, kValue,
+          UserHistoryPredictor::Entry::CLEAN_UNUSED_EVENT);
+
   Segment segment;
   segment.set_key(kKey);
   Segment::Candidate *c = segment.add_candidate();
@@ -1457,12 +1579,16 @@ TEST(UserHistoryPredictor, FingerPrintTest) {
       UserHistoryPredictor::SegmentFingerprint(segment2);
 
   EXPECT_EQ(entry_fp1, entry_fp2);
+  EXPECT_EQ(entry_fp1, entry_fp3);
+  EXPECT_NE(entry_fp1, entry_fp4);
+  EXPECT_NE(entry_fp1, entry_fp5);
+  EXPECT_NE(entry_fp4, entry_fp5);
   EXPECT_EQ(segment_fp, entry_fp2);
   EXPECT_EQ(segment_fp, entry_fp1);
   EXPECT_EQ(segment_fp, segment_fp2);
 }
 
-TEST(UserHistoryPredictor, Uint32ToStringTest) {
+TEST_F(UserHistoryPredictorTest, Uint32ToStringTest) {
   EXPECT_EQ(123,
             UserHistoryPredictor::StringToUint32(
                 UserHistoryPredictor::Uint32ToString(123)));
@@ -1484,7 +1610,7 @@ TEST(UserHistoryPredictor, Uint32ToStringTest) {
   EXPECT_EQ(0, UserHistoryPredictor::StringToUint32("abcdef"));
 }
 
-TEST(UserHistoryPredictor, GetScore) {
+TEST_F(UserHistoryPredictorTest, GetScore) {
   // latest value has higher score.
   {
     UserHistoryPredictor::Entry entry1, entry2;
@@ -1552,20 +1678,52 @@ TEST(UserHistoryPredictor, GetScore) {
   }
 }
 
-TEST(UserHistoryPredictor, IsValidSuggestion) {
+TEST_F(UserHistoryPredictorTest, IsValidEntry) {
   UserHistoryPredictor::Entry entry;
 
-  EXPECT_FALSE(UserHistoryPredictor::IsValidSuggestion(1, entry));
+  EXPECT_TRUE(UserHistoryPredictor::IsValidEntry(entry));
+
+  entry.set_key("key");
+  entry.set_key("value");
+
+  EXPECT_TRUE(UserHistoryPredictor::IsValidEntry(entry));
+
+  entry.set_removed(true);
+  EXPECT_FALSE(UserHistoryPredictor::IsValidEntry(entry));
+
+  entry.set_removed(false);
+  EXPECT_TRUE(UserHistoryPredictor::IsValidEntry(entry));
+
+  entry.set_entry_type(UserHistoryPredictor::Entry::CLEAN_ALL_EVENT);
+  EXPECT_FALSE(UserHistoryPredictor::IsValidEntry(entry));
+
+  entry.set_entry_type(UserHistoryPredictor::Entry::CLEAN_UNUSED_EVENT);
+  EXPECT_FALSE(UserHistoryPredictor::IsValidEntry(entry));
+
+  entry.set_removed(true);
+  EXPECT_FALSE(UserHistoryPredictor::IsValidEntry(entry));
+
+  entry.Clear();
+  EXPECT_TRUE(UserHistoryPredictor::IsValidEntry(entry));
+}
+
+TEST_F(UserHistoryPredictorTest, IsValidSuggestion) {
+  UserHistoryPredictor::Entry entry;
+
+  EXPECT_FALSE(UserHistoryPredictor::IsValidSuggestion(false, 1, entry));
 
   entry.set_bigram_boost(true);
-  EXPECT_TRUE(UserHistoryPredictor::IsValidSuggestion(1, entry));
+  EXPECT_TRUE(UserHistoryPredictor::IsValidSuggestion(false, 1, entry));
+
+  entry.set_bigram_boost(false);
+  EXPECT_TRUE(UserHistoryPredictor::IsValidSuggestion(true, 1, entry));
 
   entry.set_bigram_boost(false);
   entry.set_conversion_freq(10);
-  EXPECT_TRUE(UserHistoryPredictor::IsValidSuggestion(1, entry));
+  EXPECT_TRUE(UserHistoryPredictor::IsValidSuggestion(false, 1, entry));
 }
 
-TEST(UserHistoryPredictor, EntryPriorityQueueTest) {
+TEST_F(UserHistoryPredictorTest, EntryPriorityQueueTest) {
   // removed automatically
   const int kSize = 10000;
   {
@@ -1619,5 +1777,107 @@ TEST(UserHistoryPredictor, EntryPriorityQueueTest) {
     EXPECT_EQ(2, queue.size());
   }
 }
+
+TEST_F(UserHistoryPredictorTest, PrivacySensitiveTest) {
+  UserHistoryPredictor predictor;
+  predictor.WaitForSyncer();
+
+  {
+    Segments segments;
+    MakeSegmentsForConversion("123abc!", &segments);
+    AddCandidate("123abc!", &segments);
+    predictor.Finish(&segments);
+  }
+
+  // no suggestion for password-like input
+  {
+    Segments segments;
+    MakeSegmentsForSuggestion("123abc", &segments);
+    EXPECT_FALSE(predictor.Predict(&segments));
+    segments.Clear();
+    MakeSegmentsForSuggestion("123abc!", &segments);
+    EXPECT_FALSE(predictor.Predict(&segments));
+  }
+
+  // no prediction for password-like input
+  {
+    Segments segments;
+    MakeSegmentsForPrediction("123abc", &segments);
+    EXPECT_FALSE(predictor.Predict(&segments));
+    segments.Clear();
+    MakeSegmentsForPrediction("123abc!", &segments);
+    EXPECT_FALSE(predictor.Predict(&segments));
+  }
+
+  predictor.ClearAllHistory();
+  predictor.WaitForSyncer();
+  {
+    Segments segments;
+    MakeSegmentsForConversion("123", &segments);
+    MakeSegmentsForConversion("abc!", &segments);
+    AddCandidate(0, "123", &segments);
+    AddCandidate(1, "abc!", &segments);
+    predictor.Finish(&segments);
+  }
+
+  // treat as not privacy sensitive but conversion result
+  {
+    Segments segments;
+    MakeSegmentsForSuggestion("123abc", &segments);
+    EXPECT_TRUE(predictor.Predict(&segments));
+    segments.Clear();
+    MakeSegmentsForSuggestion("123abc!", &segments);
+    EXPECT_TRUE(predictor.Predict(&segments));
+  }
+
+  {
+    Segments segments;
+    MakeSegmentsForPrediction("123abc", &segments);
+    EXPECT_TRUE(predictor.Predict(&segments));
+    segments.Clear();
+    MakeSegmentsForPrediction("123abc!", &segments);
+    EXPECT_TRUE(predictor.Predict(&segments));
+  }
+
+  predictor.ClearAllHistory();
+  predictor.WaitForSyncer();
+  {
+    Segments segments;
+    // "ぐーぐる"
+    MakeSegmentsForConversion(
+        "\xE3\x81\x90\xE3\x83\xBC\xE3\x81\x90\xE3\x82\x8B",
+        &segments);
+    AddCandidate(0, "Google", &segments);
+    predictor.Finish(&segments);
+  }
+
+  // treat as not privacy sensitive but conversion result
+  {
+    Segments segments;
+    // "ぐーぐ"
+    MakeSegmentsForSuggestion(
+        "\xE3\x81\x90\xE3\x83\xBC\xE3\x81\x90", &segments);
+    EXPECT_TRUE(predictor.Predict(&segments));
+  }
+}
+
+TEST_F(UserHistoryPredictorTest, UserHistoryStorage) {
+  const string filename =
+      Util::JoinPath(Util::GetUserProfileDirectory(), "test");
+
+  UserHistoryStorage storage1(filename);
+  EXPECT_EQ(filename, storage1.filename());
+
+  UserHistoryPredictor::Entry *entry = storage1.add_entries();
+  CHECK(entry);
+  entry->set_key("key");
+  entry->set_key("value");
+  storage1.Save();
+  UserHistoryStorage storage2(filename);
+  storage2.Load();
+
+  EXPECT_EQ(storage1.DebugString(), storage2.DebugString());
+  Util::Unlink(filename);
+}
 }  // namespace
 }  // namespace mozc
