@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -30,20 +30,22 @@
 #include "rewriter/user_segment_history_rewriter.h"
 
 #include <algorithm>
+#include <cctype>
 #include <set>
 #include <string>
 #include <vector>
 
 #include "base/config_file_stream.h"
 #include "base/util.h"
-#include "transliteration/transliteration.h"
 #include "converter/character_form_manager.h"
-#include "converter/pos_matcher.h"
 #include "converter/segments.h"
-#include "storage/lru_storage.h"
+#include "dictionary/pos_matcher.h"
 #include "rewriter/rewriter_interface.h"
-#include "session/config_handler.h"
+#include "rewriter/variants_rewriter.h"
 #include "session/config.pb.h"
+#include "session/config_handler.h"
+#include "storage/lru_storage.h"
+#include "transliteration/transliteration.h"
 #include "usage_stats/usage_stats.h"
 
 namespace mozc {
@@ -52,6 +54,11 @@ namespace {
 const uint32 kValueSize = 4;
 const uint32 kLRUSize   = 20000;
 const uint32 kSeedValue = 0xf28defe3;
+const uint32 kMaxCandidatesSize = 255;
+
+const char kFileName[] = "user://segment.db";
+
+LRUStorage *g_lru_storage = NULL;
 
 class FeatureValue {
  public:
@@ -104,7 +111,7 @@ class KeyTriggerValue {
   }
 
   void set_candidates_size(uint32 size) {
-    candidates_size_ = min(size, static_cast<uint32>(256));
+    candidates_size_ = min(size, kMaxCandidatesSize);
   }
 
  private:
@@ -135,7 +142,7 @@ class ScoreTypeCompare {
 inline int GetDefaultCandidateIndex(const Segment &segment) {
   const int size = min(static_cast<int>(segment.candidates_size()), 5);
   for (int i = 0; i < size; ++i) {
-    if (segment.candidate(i).learning_type &
+    if (segment.candidate(i).attributes &
         Segment::Candidate::BEST_CANDIDATE) {
       return i;
     }
@@ -326,14 +333,14 @@ void NormalizeCandidate(const Segment *segment, int n,
   const Segment::Candidate &candidate = segment->candidate(n);
 
   // use "AS IS"
-  if (!candidate.can_expand_alternative) {
+  if (candidate.attributes & Segment::Candidate::NO_VARIANTS_EXPANSION) {
     *normalized_value = candidate.value;
     return;
   }
 
   string result = candidate.value;
   switch (candidate.style) {
-    case Segment::Candidate::DEFAULT:
+    case Segment::Candidate::DEFAULT_STYLE:
       CharacterFormManager::GetCharacterFormManager()->
           ConvertConversionString(candidate.value, &result);
       break;
@@ -381,7 +388,7 @@ bool GetSameValueCandidatePosition(const Segment *segment,
   }
   for (size_t i = 0; i < segment->meta_candidates_size(); ++i) {
     if (segment->meta_candidate(i).value == candidate->value) {
-      *position = (-i-1);  // meta candidate index
+      *position = (-static_cast<int>(i)-1);  // meta candidate index
       return true;
     }
   }
@@ -446,9 +453,11 @@ bool SortCandidates(const vector<ScoreType> &sorted_scores, Segment *segment) {
         CharacterFormManager::GetCharacterFormManager()->
             ConvertConversionString(candidate->content_value,
                                     &(new_candidate->content_value));
-        new_candidate->ResetDescription(
-            (Segment::Candidate::FULL_HALF_WIDTH |
-             Segment::Candidate::PLATFORM_DEPENDENT_CHARACTER));
+        // Update description so it matches candidate's current value.
+        // This fix addresses Bug #3493644.
+        // (Wrong character width annotation after learning alphabet)
+        new_candidate->description.clear();
+        VariantsRewriter::SetDescriptionForCandidate(new_candidate);
         ++next_pos;
         seen.insert(normalized_value);
       }
@@ -466,13 +475,8 @@ bool SortCandidates(const vector<ScoreType> &sorted_scores, Segment *segment) {
 
 UserSegmentHistoryRewriter::UserSegmentHistoryRewriter()
     : storage_(new LRUStorage) {
-  static const char kFileName[] = "user://segment.db";
-  const string filename = ConfigFileStream::GetFileName(kFileName);
-  if (!storage_->OpenOrCreate(filename.c_str(),
-                              kValueSize, kLRUSize, kSeedValue)) {
-    LOG(WARNING) << "cannot initialize UserSegmentHistoryRewriter";
-    storage_.reset(NULL);
-  }
+  Reload();
+  g_lru_storage = storage_.get();
 
   CHECK_EQ(sizeof(uint32), sizeof(FeatureValue));
   CHECK_EQ(sizeof(uint32), sizeof(KeyTriggerValue));
@@ -509,11 +513,11 @@ do { \
 bool UserSegmentHistoryRewriter::GetScore(const Segments &segments,
                                           size_t segment_index,
                                           int candidate_index,
-                                          const string &top_functional_value,
-                                          uint16 top_pos_group,
                                           uint32 *score,
                                           uint32 *last_access_time) const {
   const size_t segments_size = segments.conversion_segments_size();
+  const Segment::Candidate &top_candidate =
+      segments.segment(segment_index).candidate(0);
   const Segment::Candidate &candidate =
       segments.segment(segment_index).candidate(candidate_index);
   const string &all_value = candidate.value;
@@ -525,13 +529,9 @@ bool UserSegmentHistoryRewriter::GetScore(const Segments &segments,
   // don't apply UNIGRAM model
   const bool context_sensitive =
       segments.resized() ||
-      (candidate.learning_type & Segment::Candidate::CONTEXT_SENSITIVE) ||
-      (segments.segment(segment_index).candidate(0).learning_type &
+      (candidate.attributes & Segment::Candidate::CONTEXT_SENSITIVE) ||
+      (segments.segment(segment_index).candidate(0).attributes &
        Segment::Candidate::CONTEXT_SENSITIVE);
-  const bool is_replaceable =
-      top_functional_value == candidate.functional_value() &&
-      top_pos_group == GetPosGroup(candidate);
-
   DCHECK(score);
   DCHECK(last_access_time);
 
@@ -557,6 +557,8 @@ bool UserSegmentHistoryRewriter::GetScore(const Segments &segments,
   FETCH_FEATURE(GetFeatureLN, content_key, content_value, bigram_number_score);
   FETCH_FEATURE(GetFeatureRN, content_key, content_value, bigram_number_score);
 
+  const bool is_replaceable = Replaceable(top_candidate, candidate);
+
   if (!context_sensitive && is_replaceable) {
     FETCH_FEATURE(GetFeatureC,  all_key, all_value, unigram_score);
   }
@@ -583,6 +585,24 @@ bool UserSegmentHistoryRewriter::GetScore(const Segments &segments,
   return (*score > 0);
 }
 
+namespace {
+bool IsT13NCandidate(const Segment::Candidate &cand) {
+  // Regard the cand with 0-id as the transliterated candidate.
+  return (cand.lid == 0 && cand.rid == 0);
+}
+}  // namespace
+
+// Returns true if |lhs| candidate can be replaceable with |rhs|.
+bool UserSegmentHistoryRewriter::Replaceable(
+    const Segment::Candidate &lhs, const Segment::Candidate &rhs) const {
+  const bool same_functional_value =
+      (lhs.functional_value() == rhs.functional_value());
+  const bool same_pos_group = (GetPosGroup(lhs) == GetPosGroup(rhs));
+  return (same_functional_value &&
+          (same_pos_group || IsT13NCandidate(lhs) || IsT13NCandidate(rhs)));
+}
+
+
 void UserSegmentHistoryRewriter::RememberNumberPreference(
     const Segment &segment) {
   const Segment::Candidate &candidate = segment.candidate(0);
@@ -602,7 +622,7 @@ void UserSegmentHistoryRewriter::RememberNumberPreference(
     // separated and default is learned at same time
     // This problem is solved by workaround on lookup.
     string default_feature_key;
-    GetFeatureN(Segment::Candidate::DEFAULT, &default_feature_key);
+    GetFeatureN(Segment::Candidate::DEFAULT_STYLE, &default_feature_key);
     FeatureValue v;
     DCHECK(v.IsValid());
     storage_->Insert(default_feature_key, reinterpret_cast<const char *>(&v));
@@ -633,7 +653,7 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
   }
 
   const bool context_sensitive = segments.resized() ||
-      (candidate.learning_type & Segment::Candidate::CONTEXT_SENSITIVE);
+      (candidate.attributes & Segment::Candidate::CONTEXT_SENSITIVE);
   const string &all_value = candidate.value;
   const string &content_value = candidate.content_value;
   const string &all_key = seg.key();
@@ -642,20 +662,17 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
   // even if the candiate was the top (default) candidate,
   // ERANKED will be set when user changes the ranking
   const bool force_insert =
-      (candidate.learning_type & Segment::Candidate::RERANKED);
+      (candidate.attributes & Segment::Candidate::RERANKED);
 
   // Compare the POS group and Functional value.
   // if "is_replaceable" is true, it means that  the target candidate can
   // "SAFELY" be replaceable with the top candidate.
   const int top_index = GetDefaultCandidateIndex(seg);
-  const bool is_replaceable =
-      (top_index == 0) ||
-      (GetPosGroup(seg.candidate(top_index)) == GetPosGroup(candidate) &&
-       seg.candidate(top_index).functional_value() == candidate.functional_value());
+  const bool is_replaceable_with_top =
+      ((top_index == 0) || Replaceable(seg.candidate(top_index), candidate));
 
   // |feature_key| is used inside INSERT_FEATURE
   string feature_key;
-
   INSERT_FEATURE(GetFeatureLR, all_key, all_value, force_insert);
   INSERT_FEATURE(GetFeatureLL, all_key, all_value, force_insert);
   INSERT_FEATURE(GetFeatureRR, all_key, all_value, force_insert);
@@ -665,12 +682,14 @@ void UserSegmentHistoryRewriter::RememberFirstCandidate(
   INSERT_FEATURE(GetFeatureRN, all_key, all_value, force_insert);
   INSERT_FEATURE(GetFeatureS,  all_key, all_value, force_insert);
 
-  if (!context_sensitive && is_replaceable) {
+  if (!context_sensitive && is_replaceable_with_top) {
     INSERT_FEATURE(GetFeatureC, all_key, all_value, force_insert);
   }
 
   // save content value
-  if (all_value != content_value && all_key != content_key && is_replaceable) {
+  if (all_value != content_value &&
+      all_key != content_key &&
+      is_replaceable_with_top) {
     INSERT_FEATURE(GetFeatureLR, content_key, content_value, force_insert);
     INSERT_FEATURE(GetFeatureLL, content_key, content_value, force_insert);
     INSERT_FEATURE(GetFeatureRR, content_key, content_value, force_insert);
@@ -704,8 +723,8 @@ bool UserSegmentHistoryRewriter::IsAvailable(const Segments &segments) const {
     return false;
   }
 
-  if (!segments.use_user_history()) {
-    VLOG(2) << "no use_user_history";
+  if (!segments.user_history_enabled()) {
+    VLOG(2) << "!user_history_enabled";
     return false;
   }
 
@@ -741,9 +760,8 @@ void UserSegmentHistoryRewriter::Finish(Segments *segments) {
     const Segment &segment = segments->segment(i);
     if (segment.candidates_size() <= 0 ||
         segment.segment_type() != Segment::FIXED_VALUE ||
-        segment.candidate(0).learning_type &
+        segment.candidate(0).attributes &
         Segment::Candidate::NO_HISTORY_LEARNING) {
-      VLOG(2) << i << " cannot be remembered";
       continue;
     }
     if (IsNumberSegment(segment)) {
@@ -758,6 +776,23 @@ void UserSegmentHistoryRewriter::Finish(Segments *segments) {
                                       static_cast<int>(storage_->used_size()));
 }
 
+bool UserSegmentHistoryRewriter::Reload() {
+  const string filename = ConfigFileStream::GetFileName(kFileName);
+  if (!storage_->OpenOrCreate(filename.c_str(),
+                              kValueSize, kLRUSize, kSeedValue)) {
+    LOG(WARNING) << "cannot initialize UserSegmentHistoryRewriter";
+    storage_.reset(NULL);
+    return false;
+  }
+
+  const char kFileSuffix[] = ".merge_pending";
+  const string merge_pending_file = filename + kFileSuffix;
+  storage_->Merge(merge_pending_file.c_str());
+  Util::Unlink(merge_pending_file);
+
+  return true;
+}
+
 bool UserSegmentHistoryRewriter::ShouldRewrite(
     const Segment &segment,
     size_t *max_candidates_size) const {
@@ -788,7 +823,7 @@ bool UserSegmentHistoryRewriter::ShouldRewrite(
 }
 
 void UserSegmentHistoryRewriter::InsertTriggerKey(const Segment &segment) {
-  if (!(segment.candidate(0).learning_type & Segment::Candidate::RERANKED)) {
+  if (!(segment.candidate(0).attributes & Segment::Candidate::RERANKED)) {
     VLOG(2) << "InsertTriggerKey is skipped";
     return;
   }
@@ -875,7 +910,7 @@ bool UserSegmentHistoryRewriter::Rewrite(Segments *segments) const {
     Segment *segment = segments->mutable_segment(i);
     DCHECK(segment);
     DCHECK_GT(segment->candidates_size(), 0);
-    segment->mutable_candidate(0)->learning_type |=
+    segment->mutable_candidate(0)->attributes |=
         Segment::Candidate::BEST_CANDIDATE;
   }
 
@@ -904,19 +939,11 @@ bool UserSegmentHistoryRewriter::Rewrite(Segments *segments) const {
       continue;
     }
 
-    // max_candidates_size == 0 means that answer is in META candidate
-    static const size_t kMaxCandidatesSizeReserved = 10;
-    segment->GetCandidates(max_candidates_size + kMaxCandidatesSizeReserved);
-
     if (segment->candidates_size() < max_candidates_size) {
       LOG(WARNING) << "cannot expand candidates. ignored."
                    << "rewrite may be failed ";
     }
 
-    const string top_functional_value =
-        segment->candidate(0).functional_value();
-    const uint16 top_pos_group = GetPosGroup(segment->candidate(0));
-
     // for each all candidates expanded
     vector<ScoreType> scores;
     for (size_t l = 0;
@@ -930,8 +957,7 @@ bool UserSegmentHistoryRewriter::Rewrite(Segments *segments) const {
 
       uint32 score = 0;
       uint32 last_access_time = 0;
-      if (GetScore(*segments, i, j, top_functional_value, top_pos_group,
-                   &score, &last_access_time)) {
+      if (GetScore(*segments, i, j, &score, &last_access_time)) {
         scores.resize(scores.size() + 1);
         scores.back().score = score;
         scores.back().last_access_time = last_access_time;
@@ -955,4 +981,9 @@ void UserSegmentHistoryRewriter::Clear() {
     storage_->Clear();
   }
 }
+
+// static
+LRUStorage *UserSegmentHistoryRewriter::GetStorage() {
+  return g_lru_storage;
+}
 }  // namespace
