@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -179,6 +179,122 @@ TEST(SessionOutputTest, FillCandidates) {
   EXPECT_FALSE(candidates_proto.subcandidates().has_focused_index());
 }
 
+TEST(SessionOutputTest, FillAllCandidateWords) {
+  // IDs are ordered by BFS.
+  // 
+  //  ID|Idx| Candidate list tree
+  //   1| 0 | [1:[sub1_1,
+  //   5| 1 |    sub1_2:[subsub1_1,
+  //   6| 2 |            subsub1_2],
+  //   2| 3 |    sub1_3],
+  //   0| 4 |  2,
+  //   3| 5 |  3:[sub2_1,
+  //   4| 6 |     sub2_2]]
+  CandidateList main_list(true);
+  CandidateList sub1(true);
+  CandidateList sub2(true);
+  CandidateList subsub1(true);
+  commands::CandidateList candidates_proto;
+
+  // Initialize Segment
+  Segment segment;
+  const char* kNormalKey = "key";
+  segment.set_key(kNormalKey);
+
+  const char* kValues[7] =
+    {"2", "sub1_1", "sub1_3", "sub2_1", "sub2_2", "subsub1_1", "subsub1_2"};
+  const size_t kValueSize = arraysize(kValues);
+  for (size_t i = 0; i < kValueSize; ++i) {
+    Segment::Candidate *candidate = segment.push_back_candidate();
+    candidate->content_key = kNormalKey;
+    candidate->value = kValues[i];
+  }
+  // Set special key to ID:4 / Index:6
+  const char* kSpecialKey = "Special Key";
+  segment.mutable_candidate(4)->content_key = kSpecialKey;
+
+  // Main
+  main_list.AddSubCandidateList(&sub1);
+  main_list.AddCandidate(0, kValues[0]);
+  main_list.AddSubCandidateList(&sub2);
+
+  // Sub1
+  sub1.AddCandidate(1, kValues[1]);
+  sub1.AddSubCandidateList(&subsub1);
+  sub1.AddCandidate(2, kValues[2]);
+
+  // Sub2
+  sub2.AddCandidate(3, kValues[3]);
+  sub2.AddCandidate(4, kValues[4]);
+
+  // SubSub1
+  subsub1.AddCandidate(5, kValues[5]);
+  subsub1.AddCandidate(6, kValues[6]);
+
+  // Set forcus to ID:5 / Index:1
+  main_list.set_focused(true);
+  sub1.set_focused(true);
+  subsub1.set_focused(true);
+  main_list.MoveToId(5);
+  EXPECT_EQ(5, main_list.focused_id());
+  EXPECT_EQ(0, main_list.focused_index());
+  EXPECT_EQ(1, sub1.focused_index());
+  EXPECT_EQ(0, subsub1.focused_index());
+  // End of Initialization
+
+
+  // Exexcute FillAllCandidateWords
+  const commands::Category kCategory = commands::PREDICTION;
+  SessionOutput::FillAllCandidateWords(segment, main_list, kCategory,
+                                       &candidates_proto);
+
+  // Varidation
+  EXPECT_EQ(1, candidates_proto.focused_index());
+  EXPECT_EQ(kCategory, candidates_proto.category());
+  EXPECT_EQ(kValueSize, candidates_proto.candidates_size());
+
+  EXPECT_EQ(1, candidates_proto.candidates(0).id());
+  EXPECT_EQ(5, candidates_proto.candidates(1).id());
+  EXPECT_EQ(6, candidates_proto.candidates(2).id());
+  EXPECT_EQ(2, candidates_proto.candidates(3).id());
+  EXPECT_EQ(0, candidates_proto.candidates(4).id());
+  EXPECT_EQ(3, candidates_proto.candidates(5).id());
+  EXPECT_EQ(4, candidates_proto.candidates(6).id());
+
+  EXPECT_EQ(0, candidates_proto.candidates(0).index());
+  EXPECT_EQ(1, candidates_proto.candidates(1).index());
+  EXPECT_EQ(2, candidates_proto.candidates(2).index());
+  EXPECT_EQ(3, candidates_proto.candidates(3).index());
+  EXPECT_EQ(4, candidates_proto.candidates(4).index());
+  EXPECT_EQ(5, candidates_proto.candidates(5).index());
+  EXPECT_EQ(6, candidates_proto.candidates(6).index());
+
+  EXPECT_FALSE(candidates_proto.candidates(0).has_key());
+  EXPECT_FALSE(candidates_proto.candidates(1).has_key());
+  EXPECT_FALSE(candidates_proto.candidates(2).has_key());
+  EXPECT_FALSE(candidates_proto.candidates(3).has_key());
+  EXPECT_FALSE(candidates_proto.candidates(4).has_key());
+  EXPECT_FALSE(candidates_proto.candidates(5).has_key());
+  EXPECT_TRUE(candidates_proto.candidates(6).has_key());
+  EXPECT_EQ(kSpecialKey, candidates_proto.candidates(6).key());
+
+  EXPECT_EQ(kValues[1], candidates_proto.candidates(0).value());
+  EXPECT_EQ(kValues[5], candidates_proto.candidates(1).value());
+  EXPECT_EQ(kValues[6], candidates_proto.candidates(2).value());
+  EXPECT_EQ(kValues[2], candidates_proto.candidates(3).value());
+  EXPECT_EQ(kValues[0], candidates_proto.candidates(4).value());
+  EXPECT_EQ(kValues[3], candidates_proto.candidates(5).value());
+  EXPECT_EQ(kValues[4], candidates_proto.candidates(6).value());
+
+  EXPECT_FALSE(candidates_proto.candidates(0).has_annotation());
+  EXPECT_FALSE(candidates_proto.candidates(1).has_annotation());
+  EXPECT_FALSE(candidates_proto.candidates(2).has_annotation());
+  EXPECT_FALSE(candidates_proto.candidates(3).has_annotation());
+  EXPECT_FALSE(candidates_proto.candidates(4).has_annotation());
+  EXPECT_FALSE(candidates_proto.candidates(5).has_annotation());
+  EXPECT_FALSE(candidates_proto.candidates(6).has_annotation());
+}
+
 TEST(SessionOutputTest, FillUsages) {
   Segment segment;
   CandidateList candidate_list(true);
@@ -187,19 +303,21 @@ TEST(SessionOutputTest, FillUsages) {
   // Make 5 candidates
   static const struct DummySegment {
     const char *value;
+    const int32 usage_id;
     const char *usage_title;
     const char *usage_description;
   } dummy_segments[] = {
-    { "val0", "title0", "desc0" },
-    { "val1", "", "" },
-    { "val2", "title2", "desc2" },
-    { "val3", "title3", "desc3" },
-    { "val4", "", "" },
+    { "val0", 100, "title0", "desc0" },
+    { "val1", 0, "", "" },
+    { "val2", 200, "title2", "desc2" },
+    { "val3", 300, "title3", "desc3" },
+    { "val4", 0, "", "" },
   };
   for (size_t i = 0; i < 5; ++i) {
     Segment::Candidate *cand = segment.push_back_candidate();
     candidate_list.AddCandidate(i, dummy_segments[i].value);
     cand->value = dummy_segments[i].value;
+    cand->usage_id = dummy_segments[i].usage_id;
     cand->usage_title = dummy_segments[i].usage_title;
     cand->usage_description = dummy_segments[i].usage_description;
   }
@@ -214,21 +332,21 @@ TEST(SessionOutputTest, FillUsages) {
   EXPECT_EQ(3, candidates_proto.usages().information_size());
 
   // Usage:0 should be dummy_segment:0
-  EXPECT_EQ(0, candidates_proto.usages().information(0).id());
+  EXPECT_EQ(100, candidates_proto.usages().information(0).id());
   EXPECT_EQ(dummy_segments[0].usage_title,
             candidates_proto.usages().information(0).title());
   EXPECT_EQ(dummy_segments[0].usage_description,
             candidates_proto.usages().information(0).description());
 
   // Usage:1 should be dummy_segment:2
-  EXPECT_EQ(2, candidates_proto.usages().information(1).id());
+  EXPECT_EQ(200, candidates_proto.usages().information(1).id());
   EXPECT_EQ(dummy_segments[2].usage_title,
             candidates_proto.usages().information(1).title());
   EXPECT_EQ(dummy_segments[2].usage_description,
             candidates_proto.usages().information(1).description());
 
   // Usage:2 should be dummy_segment:3
-  EXPECT_EQ(3, candidates_proto.usages().information(2).id());
+  EXPECT_EQ(300, candidates_proto.usages().information(2).id());
   EXPECT_EQ(dummy_segments[3].usage_title,
             candidates_proto.usages().information(2).title());
   EXPECT_EQ(dummy_segments[3].usage_description,
@@ -428,5 +546,46 @@ TEST(SessionOutputTest, FillPreeditResult) {
   EXPECT_EQ("ABC", result.value());
 }
 
+TEST(SessionOutputTest, FillAllCandidateWords_NonForcused) {
+  // Test against b/3059255
+  // Even when no candidate was focused, all_candidate_words had focused_index.
+
+  CandidateList main_list(true);
+  commands::CandidateList candidates_proto;
+  main_list.AddCandidate(0, "key");
+
+  // Initialize Segment
+  Segment segment;
+  const char *kNormalKey = "key";
+  segment.set_key(kNormalKey);
+
+  Segment::Candidate *candidate = segment.push_back_candidate();
+  candidate->content_key = "key";
+  candidate->value = "value";
+
+  {
+    // Exexcute FillAllCandidateWords
+    const commands::Category kCategory = commands::SUGGESTION;
+    SessionOutput::FillAllCandidateWords(segment, main_list, kCategory,
+                                         &candidates_proto);
+
+    // Varidation
+    EXPECT_FALSE(candidates_proto.has_focused_index());
+  }
+  {
+    main_list.set_focused(true);
+    // Exexcute FillAllCandidateWords
+    // When the category is SUGGESTION, has_focused_index never return true in
+    // real usage. This is just a testcase.
+    const commands::Category kCategory = commands::SUGGESTION;
+    SessionOutput::FillAllCandidateWords(segment, main_list, kCategory,
+                                         &candidates_proto);
+
+    // Varidation
+    // If a candidate is forcused, true is expected.
+    EXPECT_TRUE(candidates_proto.has_focused_index());
+  }
+}
+
 }  // namespace session
 }  // namespace mozc
