@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,8 @@
 
 #ifdef OS_WINDOWS
 #include <windows.h>
+#else
+#include <unistd.h>
 #endif  // OS_WINDOWS
 
 #include <climits>
@@ -66,7 +68,9 @@ string GetProgramPath() {
 }
 
 string GetTestSrcdir() {
-  return MOZC_DATA_DIR;
+  // Return program path in default.
+  // TODO(toshiyuki) : Make it to be configured in build stage
+  return GetProgramPath();
 }
 
 string GetTestTmpdir() {
