@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2011, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -43,6 +43,7 @@
 #include "session/commands.pb.h"
 #include "session/config.pb.h"
 #include "session/ime_switch_util.h"
+#include "unix/ibus/config_util.h"
 #include "unix/ibus/engine_registrar.h"
 #include "unix/ibus/key_translator.h"
 #include "unix/ibus/mozc_engine_property.h"
@@ -77,6 +78,34 @@ const guint kBackSpaceKeyCode = 14;
 // Left shift key code
 const guint kShiftLeftKeyCode = 42;
 
+#ifdef OS_CHROMEOS
+// IBus config names for Mozc.
+// This list must match the preference names of ibus-mozc for Chrome OS.
+// Check the following file in the chrome repository when you modify this list.
+// chrome/browser/chromeos/language_preferences.cc
+const gchar* kMozcConfigNames[] = {
+  "incognito_mode",
+  "use_auto_ime_turn_off",
+  "use_date_conversion",
+  "use_single_kanji_conversion",
+  "use_symbol_conversion",
+  "use_number_conversion",
+  "use_history_suggest",
+  "use_dictionary_suggest",
+  "use_realtime_conversion",
+  "preedit_method",
+  "session_keymap",
+  "punctuation_method",
+  "symbol_method",
+  "space_character_form",
+  "history_learning_level",
+  //  "selection_shortcut",  // currently not supported
+  "shift_key_mode_switch",
+  "numpad_character_form",
+  "suggestions_size",
+};
+#endif  // OS_CHROMEOS
+
 uint64 GetTime() {
   return static_cast<uint64>(time(NULL));
 }
@@ -87,60 +116,6 @@ bool IsMozcToolAvailable() {
       mozc::Util::JoinPath(mozc::Util::GetServerDirectory(), mozc::kMozcTool));
 }
 
-#ifdef OS_CHROMEOS
-#if IBUS_CHECK_VERSION(1, 3, 99)
-// IBus-1.4 uses Glib's GVariant for configration.
-bool GetStringConfig(GVariant *value, const gchar **out_string) {
-  if (g_variant_classify(value) != G_VARIANT_CLASS_STRING) {
-    return false;
-  }
-  *out_string = g_variant_get_string(value, NULL);
-  return true;
-}
-
-bool GetIntegerConfig(GVariant *value, gint *out_integer) {
-  if (g_variant_classify(value) != G_VARIANT_CLASS_INT32) {
-    return false;
-  }
-  *out_integer = g_variant_get_int32(value);
-  return true;
-}
-
-bool GetBooleanConfig(GVariant *value, gboolean *out_boolean) {
-  if (g_variant_classify(value) != G_VARIANT_CLASS_BOOLEAN) {
-    return false;
-  }
-  *out_boolean = g_variant_get_boolean(value);
-  return true;
-}
-#else
-// IBus-1.2 and 1.3 use GValue for configration.
-bool GetStringConfig(GValue *value, const gchar **out_string) {
-  if (!G_VALUE_HOLDS_STRING(value)) {
-    return false;
-  }
-  *out_string = g_value_get_string(value);
-  return true;
-}
-
-bool GetIntegerConfig(GValue *value, gint *out_integer) {
-  if (!G_VALUE_HOLDS_INT(value)) {
-    return false;
-  }
-  *out_integer = g_value_get_int(value);
-  return true;
-}
-
-bool GetBooleanConfig(GValue *value, gboolean *out_boolean) {
-  if (!G_VALUE_HOLDS_BOOLEAN(value)) {
-    return false;
-  }
-  *out_boolean = g_value_get_boolean(value);
-  return true;
-}
-#endif
-#endif  // OS_CHROMEOS
-
 struct IBusMozcEngineClass {
   IBusEngineClass parent;
 };
@@ -417,16 +392,12 @@ MozcEngine::MozcEngine()
   }
 #endif
 
-  // We don't enable the DELETE_PRECEDING_TEXT feature on Chrome OS for now
-  // since it's highly experimental.
-#ifndef OS_CHROMEOS
   // Currently client capability is fixed.
   commands::Capability capability;
   capability.set_text_deletion(commands::Capability::DELETE_PRECEDING_TEXT);
   session_->set_client_capability(capability);
   // TODO(yusukes): write a unit test to check if the capability is set
   // as expected.
-#endif
 }
 
 MozcEngine::~MozcEngine() {
@@ -512,11 +483,13 @@ void MozcEngine::FocusOut(IBusEngine *engine) {
 }
 
 void MozcEngine::PageDown(IBusEngine *engine) {
-  // TODO(mazda): Implement this.
+  // TODO(mazda,yusukes): Implement this to support arrow icons inside the Gtk+
+  // candidate window.
 }
 
 void MozcEngine::PageUp(IBusEngine *engine) {
-  // TODO(mazda): Implement this.
+  // TODO(mazda,yusukes): Implement this to support arrow icons inside the Gtk+
+  // candidate window.
 }
 
 gboolean MozcEngine::ProcessKeyEvent(
@@ -571,6 +544,8 @@ gboolean MozcEngine::ProcessKeyEvent(
     return FALSE;
   }
 
+  key.set_mode(current_composition_mode_);
+
   commands::Output output;
   if (!session_->SendKey(key, &output)) {
     LOG(ERROR) << "SendKey failed";
@@ -655,7 +630,7 @@ void MozcEngine::PropertyActivate(IBusEngine *engine,
 
 #ifndef OS_CHROMEOS
   if (prop_mozc_tool_) {
-    while (prop = ibus_prop_list_get(prop_mozc_tool_->sub_props, i++)) {
+    while ((prop = ibus_prop_list_get(prop_mozc_tool_->sub_props, i++))) {
       if (!g_strcmp0(property_name, prop->key)) {
         const MozcEngineToolProperty *entry =
             reinterpret_cast<const MozcEngineToolProperty*>(
@@ -675,7 +650,7 @@ void MozcEngine::PropertyActivate(IBusEngine *engine,
   }
 
   i = 0;
-  while (prop = ibus_prop_list_get(prop_composition_mode_->sub_props, i++)) {
+  while ((prop = ibus_prop_list_get(prop_composition_mode_->sub_props, i++))) {
     if (!g_strcmp0(property_name, prop->key)) {
       const MozcEngineProperty *entry =
           reinterpret_cast<const MozcEngineProperty*>(
@@ -700,12 +675,12 @@ void MozcEngine::PropertyActivate(IBusEngine *engine,
 
 void MozcEngine::PropertyHide(IBusEngine *engine,
                               const gchar *property_name) {
-  // TODO(mazda): Implement this.
+  // We can ignore the signal.
 }
 
 void MozcEngine::PropertyShow(IBusEngine *engine,
                               const gchar *property_name) {
-  // TODO(mazda): Implement this.
+  // We can ignore the signal.
 }
 
 void MozcEngine::Reset(IBusEngine *engine) {
@@ -724,7 +699,9 @@ void MozcEngine::Reset(IBusEngine *engine) {
 
 void MozcEngine::SetCapabilities(IBusEngine *engine,
                                  guint capabilities) {
-  // TODO(mazda): Implement this.
+  // TODO(mazda,yusukes): For now, there's nothing to do here. If mozc starts to
+  // support the reconversion feature, we should checkIBUS_CAP_SURROUNDING_TEXT
+  // here to see if the current client can provide surrounding text information.
 }
 
 void MozcEngine::SetCursorLocation(IBusEngine *engine,
@@ -732,7 +709,7 @@ void MozcEngine::SetCursorLocation(IBusEngine *engine,
                                    gint y,
                                    gint w,
                                    gint h) {
-  // TODO(mazda): Implement this.
+  // We can ignore the signal.
 }
 
 GType MozcEngine::GetType() {
@@ -791,6 +768,20 @@ void MozcEngine::ConfigValueChanged(IBusConfig *config,
   engine->UpdateConfig(section, name, value);
 }
 #endif
+
+// TODO(mazda): Move the impelementation to an appropriate file.
+void MozcEngine::InitConfig(IBusConfig *config) {
+  map<string, const char*> name_to_field;
+  for (size_t i = 0; i < arraysize(kMozcConfigNames); ++i) {
+    // Mozc uses identical names for ibus config names and protobuf config
+    // field names.
+    name_to_field.insert(make_pair(kMozcConfigNames[i], kMozcConfigNames[i]));
+  }
+  // Initialize the mozc config with the config loaded from ibus-memconf, which
+  // is the primary config storage on Chrome OS.
+  ConfigUtil::InitConfig(config, kMozcSectionName, name_to_field);
+}
+
 #endif  // OS_CHROMEOS
 
 // static
@@ -870,6 +861,7 @@ bool MozcEngine::UpdateAll(IBusEngine *engine, const commands::Output &output) {
   if (output.has_mode()) {
     UpdateCompositionMode(engine, output.mode());
   }
+  LaunchTool(output);
   return true;
 }
 
@@ -907,7 +899,6 @@ bool MozcEngine::UpdateCandidates(IBusEngine *engine,
     return true;
   }
 
-  const guint kPageSize = 9;
   const gboolean kRound = TRUE;
   const commands::Candidates &candidates = output.candidates();
   const gboolean cursor_visible = candidates.has_focused_index() ?
@@ -920,12 +911,23 @@ bool MozcEngine::UpdateCandidates(IBusEngine *engine,
       }
     }
   }
-  IBusLookupTable *table = ibus_lookup_table_new(kPageSize,
+
+  size_t page_size = kPageSize;
+  if (candidates.has_category() &&
+      candidates.category() == commands::SUGGESTION &&
+      page_size > candidates.candidate_size()) {
+    page_size = candidates.candidate_size();
+  }
+  IBusLookupTable *table = ibus_lookup_table_new(page_size,
                                                  cursor_pos,
                                                  cursor_visible,
                                                  kRound);
 #if IBUS_CHECK_VERSION(1, 3, 0)
-  ibus_lookup_table_set_orientation(table, IBUS_ORIENTATION_VERTICAL);
+  if (candidates.direction() == commands::Candidates::VERTICAL) {
+    ibus_lookup_table_set_orientation(table, IBUS_ORIENTATION_VERTICAL);
+  } else {
+    ibus_lookup_table_set_orientation(table, IBUS_ORIENTATION_HORIZONTAL);
+  }
 #endif
 
   for (int i = 0; i < candidates.candidate_size(); ++i) {
@@ -1026,74 +1028,15 @@ void MozcEngine::UpdateConfig(const gchar *section,
   }
 
   config::Config mozc_config;
-  const google::protobuf::Descriptor *descriptor = mozc_config.GetDescriptor();
-  const google::protobuf::Reflection *reflection = mozc_config.GetReflection();
-  const google::protobuf::FieldDescriptor *field_to_update =
-      descriptor->FindFieldByName(name);
-
-  if (!field_to_update) {
-    LOG(ERROR) << "Unknown config name: " << name;
-    return;
-  }
-
-  // Set |value| to |mozc_config|.
-  switch (field_to_update->cpp_type()) {
-    case google::protobuf::FieldDescriptor::CPPTYPE_ENUM: {
-      // |value| should be STRING.
-      const gchar *string_value = NULL;
-      if (!GetStringConfig(value, &string_value)) {
-        LOG(ERROR) << "Bad value type for " << name;
-        return;
-      }
-      DCHECK(string_value);
-      const google::protobuf::EnumValueDescriptor *enum_value =
-          descriptor->FindEnumValueByName(string_value);
-      if (!enum_value) {
-        LOG(ERROR) << "Bad value for " << name << ": " << string_value;
-        return;
-      }
-      reflection->SetEnum(&mozc_config, field_to_update, enum_value);
-      VLOG(2) << "setting mozc config: " << name << " = " << string_value;
-      break;
-    }
-    case google::protobuf::FieldDescriptor::CPPTYPE_UINT32: {
-      // unsigned int is not supported as chrome's preference type and int is
-      // used as an alternative type, so |value| should be INT.
-      gint int_value = -1;
-      if (!GetIntegerConfig(value, &int_value)) {
-        LOG(ERROR) << "Bad value type for " << name;
-        return;
-      }
-      reflection->SetUInt32(&mozc_config, field_to_update, int_value);
-      VLOG(2) << "setting mozc config: " << name << " = " << int_value;
-      break;
-    }
-    case google::protobuf::FieldDescriptor::CPPTYPE_BOOL: {
-      // |value| should be BOOLEAN.
-      gboolean boolean_value = FALSE;
-      if (!GetBooleanConfig(value, &boolean_value)) {
-        LOG(ERROR) << "Bad value type for " << name;
-        return;
-      }
-      reflection->SetBool(&mozc_config, field_to_update, boolean_value);
-      VLOG(2) << "setting mozc config: " << name << " = "
-              << (boolean_value ? "true" : "false");
-      break;
-    }
-    default: {
-      // TODO(yusukes): Support other types.
-      LOG(ERROR) << "Unknown or unsupported type: " << name << ": "
-                 << field_to_update->cpp_type();
-      return;
-    }
-  }
+  session_->GetConfig(&mozc_config);
+  ConfigUtil::SetFieldForName(name, value, &mozc_config);
 
   // Update config1.db.
   session_->SetConfig(mozc_config);
   session_->SyncData();  // TODO(yusukes): remove this call?
   VLOG(2) << "Session::SetConfig() is called: " << name;
 }
-#endif
+#endif  // OS_CHROMEOS
 
 void MozcEngine::UpdateCompositionMode(
     IBusEngine *engine, const commands::CompositionMode new_composition_mode) {
@@ -1133,6 +1076,15 @@ void MozcEngine::SyncData(bool force) {
   }
 }
 
+bool MozcEngine::LaunchTool(const commands::Output &output) const {
+  if (!session_->LaunchToolWithProtoBuf(output)) {
+    VLOG(2) << output.DebugString() << " Launch Failed";
+    return false;
+  }
+
+  return true;
+}
+
 void MozcEngine::RevertSession(IBusEngine *engine) {
   const commands::CompositionMode original_composition_mode =
       current_composition_mode_;
